package org.datamanager.collector.simpleton;

import org.datamanager.kernel.Simpleton;
import org.datamanager.kernel.Entity;
import org.datamanager.kernel.EntityValue;
import org.datamanager.kernel.Pool;
import org.datamanager.kernel.EventHandler;
import org.datamanager.constraint.EventConstraint;
import org.datamanager.constraint.EventTypeIsConstraint;
import org.datamanager.exception.DataManagerException;
import org.datamanager.passiveentityvalue.StringEntityValue;
import org.datamanager.passiveentityvalue.VectorEntityValue;
import org.datamanager.passiveentityvalue.ObjectEntityValue;
import org.datamanager.passiveentityvalue.IntegerEntityValue;
import org.datamanager.passiveentityvalue.DateEntityValue;
import org.datamanager.event.DataManagerEvent;
import org.datamanager.event.EmailParametersEnteredEvent;
import org.datamanager.collector.helpers.email.EmailParametersInputWindow;
import org.datamanager.collector.helpers.email.UrlParser;
import org.datamanager.collector.helpers.email.MessageFlags;
import org.datamanager.tools.Information;
import org.datamanager.tools.Debug;

import javax.mail.Session;
import javax.mail.Store;
import javax.mail.Folder;
import javax.mail.FetchProfile;
import javax.mail.Message;
import javax.mail.Address;
import javax.mail.Flags;
import javax.mail.Authenticator;
import javax.mail.AuthenticationFailedException;
import javax.mail.MessagingException;
import javax.mail.NoSuchProviderException;

import java.util.Properties;
import java.util.Vector;
import java.io.IOException;

/**
Author: Manoj Shanbhag (mshanbha@cs.indiana.edu)
Group : Networks
Date  : Nov 22, 1998
Modified on : Nov 28 ( getting it to work :) )

Collaborators:
EmailParametersInputWindow, EmailParametersEnteredEvent, MessageFlags, UrlParser

Responsibilities:
This simpleton creates an Entity representing an email message,
creates and attaches various new Entities to store attributes of that message,
then adds all the created Entities to the Pool.

NOTE: to find flaws and possible improvement ideas, search for "NOTE:".
**/

public class EmailHandlerSimpleton
	extends Simpleton
	implements EventHandler
	{
	private Entity currentEntity;
	private Pool pool;

	/**
	Report information about this simpleton.
	**/
	public Information getInformation()
		{
		String[] authors = {"Manoj Shanbhag", "gregory j. e. rawlins"};
		String name = "EmailHandlerSimpleton";
		String shortDescription =
			"This simpleton fetches email into KnownSpace.";
		String longDescription =
			"This simpleton creates entities representing " +
			"email messages, creates and attaches various " +
			"new entities to store attributes of those " +
			"messages, and adds all the created entities " +
			"to the default pool." +
			"\n\n" +
			"It creates an EmailParametersInputWindow for " +
			"the user to enter email connection parameters, " +
			"subscribes to EmailParametersEnteredEvents, " +
			"which the window generates, and uses " +
			"MessageFlags and UrlParser.";
		String version = "-1";
		Information information =
			new Information(name, shortDescription,
			longDescription, version, authors);

		return information;
		}

	/**
	Display a window for the user to enter email connection information
	and subscribe to the pool to listen for the event signaling completion.
	As a side-effect, this method sets the default pool for all other
	methods.
	**/
	public void process()
		{
		EmailParametersInputWindow parametersWindow =
			new EmailParametersInputWindow();
		parametersWindow.show();

		try
			{
			pool = Pool.getDefaultPool();
			EventConstraint eventConstraint =
				new EventTypeIsConstraint(EmailParametersEnteredEvent.class);
			pool.subscribe(this, eventConstraint);
			}
		catch (DataManagerException exception)
			{
			Debug.assert("error in kernel!");
			exception.printStackTrace();
			}
		}

	/**
	This method is still too damned long.
	It setsup the email connection parameters, setsup up JavaMail,
	makes a connection, fetches the mailbox, and parcels out each
	message to different methods that extract various bits of
	information and create entities to store them.
	**/
	public void handle(DataManagerEvent dataManagerEvent)
		{
		//extract email connection parameters
		EmailParametersEnteredEvent event =
			(EmailParametersEnteredEvent) dataManagerEvent;
		String username = event.getUsername();
		String password = event.getPassword();
		String hostMachineName = event.getHostMachineName();
		String accessProtocolName = event.getAccessProtocolName();
		String mailboxName = event.getMailboxName();

		//setup to use JavaMail
		Properties properties = System.getProperties();
		Authenticator authenticator = null; //NOTE: no checking for now
		Session session =
			Session.getDefaultInstance(properties, authenticator);
		Store store = null;
		try
			{
			if (accessProtocolName == null)
				store = session.getStore();
			else
				store = session.getStore(accessProtocolName);

			//connect to the remote machine holding the email
			//NOTE: I DON'T UNDERSTAND WHY THIS IS SO. GJER 6|12|99
			if ((hostMachineName != null) || (username != null) ||
				(password != null))
				store.connect(hostMachineName, username, password);
			else
				store.connect();

			//open the mailbox
			Folder mailbox = store.getDefaultFolder();
			if (mailbox == null)
				{
				Debug.assert("Could not find default mailbox");
				//NOTE: this message should be in a window
				store.close();
				return;
				}

			mailbox = mailbox.getFolder(mailboxName);
			if (mailbox == null)
				{
				Debug.assert("Could not find specified mailbox");
				//NOTE: this message should be in a window
				store.close();
				return;
				}

			mailbox.open(Folder.READ_ONLY);
			if (mailbox.getMessageCount() == 0)
				Debug.assert("Specified mailbox is empty");
			else
				{
				Message[] messages = mailbox.getMessages();

				// Use a suitable FetchProfile
				FetchProfile fetchProfile = new FetchProfile();
				fetchProfile.add(FetchProfile.Item.ENVELOPE);
				fetchProfile.add(FetchProfile.Item.FLAGS);
				mailbox.fetch(messages, fetchProfile);

				//NOTE: message numbers aren't cumulative;
				//they restart from 1 every time
				for (int i = 0; i < messages.length; i++)
					storeMessage(messages[i], i + 1);
				}

			mailbox.close(false);
			store.close();
			}
		catch (AuthenticationFailedException exception)
			{
			Debug.assert("Could not authenticate your information");
			//NOTE: should put retry code here
			}
		catch (NoSuchProviderException exception)
			{
			Debug.assert("Could not find host machine");
			//NOTE: should put retry code here
			}
		catch (IOException exception)
			{
			Debug.assert("Could not access stream message data");
			exception.printStackTrace();
			}
		catch (MessagingException exception)
			{
			Debug.assert("oops. something unexpected happened");
			exception.printStackTrace();
			}
		catch (DataManagerException exception)
			{
			Debug.assert("bug in kernel!");
			exception.printStackTrace();
			}
		}

	/**
	Store an email message as an entity.
	As a side-effect, this method continually resets the current entity
	for all the other store methods.
	**/
	private void storeMessage(Message message, int messageNumber)
		throws MessagingException, IOException, DataManagerException
		{
		currentEntity = Entity.create("email message " + messageNumber);
		storeEnvelopeAttributes(message);
		storeMessageFlags(message);
		storeBodyAttributes(message);
		pool.add(currentEntity);
		}

	/**
	Attach the "from", "to", "subject", "date", and "size" attributes.
	**/
	private void storeEnvelopeAttributes(Message message)
		throws MessagingException, DataManagerException
		{
		Address[] addresses = message.getFrom();
		if (addresses != null)
			{
			String senders = "";
			for (int i = 0; i < addresses.length; i++)
				senders += addresses[i].toString() + ",";

			attachStringAttribute("email senders", senders);
			}

		addresses = message.getRecipients(Message.RecipientType.TO);
		if (addresses != null)
			{
			String recipients = "";
			for (int i = 0; i < addresses.length; i++)
				recipients += addresses[i].toString() + ",";

			attachStringAttribute("email recipients", recipients);
			}

		attachStringAttribute("email subject", message.getSubject());
		attachAttribute("email date", new DateEntityValue(message.getSentDate()));
		attachAttribute("email size", new IntegerEntityValue(message.getSize()));
		}

	/**
	Attach the email message's flags
	(which designate its processed state).
	**/
	private void storeMessageFlags(Message message)
		throws MessagingException, DataManagerException
		{
		Flags flags = message.getFlags();
		MessageFlags messageFlags = new MessageFlags(flags);
		attachAttribute("email message flags", messageFlags);
		}

	/**
	Add the "content type", "content", and for plain text messages,
	extract all urls within the email message body.
	**/
	private void storeBodyAttributes(Message message)
		throws MessagingException, IOException, DataManagerException
		{
		String contentType = message.getContentType();
		attachStringAttribute("email content type", contentType);

		Object content = message.getContent();
		ObjectEntityValue contentEntityValue =
			new ObjectEntityValue(content);
		attachAttribute("email message body", contentEntityValue);

		//for plain text only: attach any urls in the message body
		if (message.isMimeType("text/plain") &&
			(content instanceof String))
			{
			UrlParser urlParser = new UrlParser();
			Vector urls = urlParser.getURLs((String) content);
			VectorEntityValue urlsEntityValue =
				new VectorEntityValue(urls);
			attachAttribute("email message urls", urlsEntityValue);
			}
		}

	/**
	Create and attach a new entity containing a string
	as an attribute of the current email message entity.
	**/
	private void attachStringAttribute(String name, String string)
		throws DataManagerException
		{
		StringEntityValue value = new StringEntityValue(string);
		attachAttribute(name, value);
		}

	/**
	Create and attach a new entity
	as an attribute of the current email message entity.
	**/
	private void attachAttribute(String name, EntityValue value)
		throws DataManagerException
		{
		Entity attributeEntity = Entity.create(name);
		attributeEntity.setValue(value);
		currentEntity.attach(attributeEntity);
		pool.add(attributeEntity);
		}
	}
