package org.datamanager.engine;

import java.io.*;
import java.util.*;


import org.datamanager.kernel.Entity;
import org.datamanager.kernel.EntityValue;

import org.datamanager.engine.cluster.*;
import org.datamanager.engine.filter.*;
import org.datamanager.engine.tools.*;

/**
 * This singleton provides engine functionality to other parts of KnownSpace.
 *
 * @author Aleksander Slominski
 */

public class Engine implements ClusterManager, EntityValue {
    public final static String ENGINE_NAME = "engine-orukter";
    public final static String ENGINE_DOCUMENT = "engine-document";

    private static Engine _instance = new Engine();
    /** @serial */
    private final EngineManager em = EngineManager.getDefault();

    Engine() {
    }

   /**
    * Get singleton.
    */
   public static Engine getDefault() {
	return _instance;
    }

    /*
    public static void setDefault(Engine e) {
	_instance = e;
	EngineManager.setDefault(e.em);
    }
    */

    /**
     * Create a cluster that engine may use.
     * The cluster is not automatically used to list of engine clusters.
     *
     * @see #addClusters
     *
     */
    public Cluster createCluster() {
	return em.createEngineCluster();
    }


    /**
     * Return list of all clusters managed by engine.
     */
    public Cluster[] getClusters() {
	synchronized(Engine.class) {
	    //em.getClusters()
	    Cluster[] cl = new EngineCluster[em.getClustersSize()];
	    for(int i = 0; i < cl.length; ++i) {
		cl[i] = em.getCluster(i);
	    }
	    return cl;
	}
    }

    /**
     * Add clusters to list of all clusters managed by engine.
     */
    public void addClusters(Cluster[] cl) {
	synchronized(Engine.class) {
	    for(int i = 0; i < cl.length; ++i)
		em.addCluster(cl[i]);
	}
    }

    /**
     * Remove clusters from list of clusters managed by engine.
     */
    public void removeClusters(Cluster[] cl) {
	synchronized(Engine.class) {
	    for(int i = 0; i < cl.length; ++i)
		em.removeCluster((EngineCluster)cl[i]);
	}
    }

    /**
     * Return what is simalirity threshold to use when clustering.
     *
     * @see #setSimilarityThreshold
     */
    public double getSimilarityThreshold() {
	return em.getSimilarityThreshold();
    }

    /**
     * Set similarity threshold to use when clustering.
     *
     * Similarity is number bigger than 0 and less or equal 1.
     * This value desribe cosinus between 2 documents. Closer they
     * are the angle approaches 0 and cosinus goest closer to 1.
     * Value of 1 means to consided only documents identical
     * (up to filtering) for clustering.
     *
     */
    public void  setSimilarityThreshold(double t) {
	em.setSimilarityThreshold(t);
    }

    /**
     * Get filter used when preparing documents content for clustring.
     */
    public Filter getFilter() {
	return em.getFilter();
    }

    /**
     * Set filter used when preparing documents content for clustring.
     */
    public void setFilter(Filter f) {
	em.setFilter(f);
    }

    /**
     * Return list of entities that each contains all keywords.
     * Or empyt list if nothing found.
     */
    public Entity[] search(String keywords[]) {
	synchronized(Engine.class) {
	    EngineCluster ec = em.search(keywords);
	    Entity[] el = new Entity[ec.size()];
	    for(int i = 0; i < el.length; ++i) {
		el[i] = ec.getDocument(i).getEntity();
	    }
	    return el;
	}
    }


    /**
     * Get list of entites that are similair to given list with given
     * similarity threshold.
     *
     */
    public Entity[] findSimilair(Entity[] list, double threshold) {
	Assertion.check(0 < threshold && threshold <= 1);
	Cluster cluster = Engine.getDefault().createCluster();
	synchronized(Engine.class) {
	  for(int k = 0; list != null && k < list.length; ++k) {
	    if(list[k] == null)
	    	continue;
	    //Entity[] el = list[k].getAttributes(new NameContainsConstraint(Engine.ENGINE_DOCUMENT));
	    //if(el == NULL || el.length == 0) {
	    //  trace("entity "+list[k]+" has no "+Engine.ENGINE_DOCUMENT+" attribute");
	    //  continue;
	    //}
	    //EngineDocument ed = (EngineDocument) el[0].getValue();
	    EngineDocument ed = em.getDocumentFromEntity(list[k]);
	    for(Iterator i = em.getDocuments(); i.hasNext(); ) {
	      EngineDocument other = (EngineDocument) i.next();
	      //trace("similarity "+ed.getUrl()+" to "+other.getUrl()+" is="+ed.getSimilarityTo(other));
	      if(ed.getSimilarityTo(other) > threshold) {
		    cluster.addEntities(new Entity[]{other.getEntity()});
	      }
	    }
	  }
	}
	return cluster.getEntities();
    }

    /**
     * Make sure that when writing we preserve singleton.
     */
    private void writeObject(ObjectOutputStream stream) throws IOException {
	Assertion.check(this == getDefault());
	stream.defaultWriteObject();
	stream.writeObject(_instance);
	trace("engine serialized");
    }

    /**
     * Restore engine, make sure we get singleton back.
     */
    private void readObject(ObjectInputStream stream) throws IOException, ClassNotFoundException {
	stream.defaultReadObject();
	_instance = (Engine) stream.readObject();
	//_instance = this;
	trace("engine deserialized");
    }

    private void trace(String s) {
	Trace.println(this.getClass(), s);
    }
}
