/**
 * WebSearchEvent.java
 */

package org.datamanager.event;

import org.datamanager.kernel.EventGenerator;
import org.datamanager.event.GenericEvent;
import org.datamanager.event.CollectorEvent;

/**
 * This event is used to trigger a web search. It has a
 * number of constants that should be used as the value
 * of "engine" in this class' constructor.
 *
 * @author Matthew Farrellee
 */
public class WebSearchEvent
   extends GenericEvent
   implements CollectorEvent
   {
   
   /* Possible Search Engines that can be used. */
   // Their format should be such that keywords can
   // be appended to the end of their url.
   public static final String WEBCRAWLER =
      "http://www.webcrawler.com/cgi-bin/WebQuery?" +
      "start=1&showSummary=false&perPage=100&search=";

   public static final String GOOGLE =
      "http://www.google.com/search?num=10&q=";

   public static final String EXCITE =
      "http://search.excite.com/search.gw?FL_1=1&FT_1=w" +
      "&FL_2=%2D1&FT_2=p&FI_2=&FL_3=0&FT_3=p&FI_3=&mode=" +
      "advanced&numFields=3&layout=default.advanced&lang=" +
      "en&c_type=c%5Fcountry&c_country=web&c_typeofsite=web" +
      ".com&sort=relevance&showSummary=false&perPage=100&" +
      "advanced=Search&c=web&FI_1=";

   public static final String DEJANEWS =
      "http://www.deja.com/[ST_rn=ps]/qs.xp?ST=PS&svcclass=" +
      "dnyr&defaultOp=AND&DBS=1&OP=dnquery.xp&LNG=ALL&" +
      "subjects=&groups=&authors=&fromdate=&todate=&showsort" +
      "=score&maxhits=100&QRY=";

   public static final String YAHOO =
      "http://search.yahoo.com/search?o=1&d=y&za=default&h=c" +
      "&g=0&n=100&p=";

   public static final String LYCOS =
      "http://lycospro.lycos.com/cgi-bin/pursuit?mtemp=nojava&" +
      "etemp=error_nojava&rt=0&npl=matchmode%3Dand%26adv%3D1&" +
      "&maxhits=100&cat=lycos&npl1=ignore%3Dfq&fq=&lang=&npl_rtwm=" +
      "&npl_rtpy=&npl_rttf=&npl_rtfd=&npl_rtpn=&npl_rtor=&query=";

   public static final String ALTAVISTA =
      "http://www.altavista.com/cgi-bin/query?pg=q&sc=on&kl=XX&" +
      "stype=stext&q=";

   private String engine;
   private String keywords;

   /**
    * The non-search engine specific constructor.
    * This constructor uses GOOGLE as the default engine.
    *
    * @param source The source of the event.
    * @param keywords The keywords with which to search.
    */
   public WebSearchEvent(EventGenerator source,
                        String keywords)
      {
      this(source, keywords, GOOGLE);
      }

   /**
    * The search engine specific consructor.
    * This constructor allows you to speficy which search
    * engine should be used in the search. The engine should
    * be one of the constants provided on this class.
    *
    * @param source The source of the event.
    * @param keywords The keywords with which to search.
    * @param engine The search engine to use for the search.
    */
   public WebSearchEvent(EventGenerator source,
                        String keywords,
                        String engine)
      {
      super(source);
      this.engine = engine;
      this.keywords = keywords;
      }


   /**
    * Get a string representation of the search engine's
    * url that is used for this search event.
    */
   public String getEngine()
      {
      return engine;
      }


   /**
    * Get the keywords used for this search event.
    */
   public String getKeywords()
      {
      return keywords;
      }


   /**
    * Get a String representation of this event.
    */
   public String toString()
      {
      return "WebSearchEvent[" +
         "[engine: " + engine + "]" +
         "[keywords: " + keywords + "]" +
         "[time: " + time + "]";
      }
   }
