/**
 * Entity.java
 *
 * Created 199909221440
 */

package org.datamanager.kernel;

import org.datamanager.kernel.EntityValue;
import org.datamanager.constraint.EntityConstraint;
import org.datamanager.constraint.EventConstraint;
import org.datamanager.exception.DataManagerException;

import java.io.Serializable;

/**
 * The major workhorse of the DataManager, Entity is an abstract class but
 * contains a lot of method implementations.
 *
 * @author Benjamin Keil
 * @author Matthew Farrellee
 */
public abstract class Entity implements EventGenerator, Serializable
   {
   ///////////////////////////////////////////////////////////////////////
   //
   //                  Entity Creation
   //
   ///////////////////////////////////////////////////////////////////////

   /**
    * Return a newly created Entity with the given name.
    */
   public synchronized static Entity create(String name)
         {
         return EntityManager.getInstance().createEntity(name);
         }

   /**
    * This constructor exists only for the sake of subclasses.
    */
   protected Entity()
      {
      }

   ///////////////////////////////////////////////////////////////////////
   //
   //                   Entity Linkage
   //
   ///////////////////////////////////////////////////////////////////////

   /**
    * Attach the given Entity as an attribute.  Relies on a private method
    * call to make a reciprocating link back from the attribute.
    */
   public abstract void attach(Entity attribute)
         throws DataManagerException;

   /**
    * Add an Entity to the list of bases.  This should only be called when
    * the base is adding this Entity as an attribute, that is, you should
    * probably *NEVER* make an explicit call to this function.
    */
   protected abstract void addBase(Entity base);

   /**
    * Detach the given Entity.
    */
   public abstract void detach(Entity attribute)
         throws DataManagerException;

   /**
    * Remove an Entity from the list of bases.  This should only be called
    * by the base as it removes this Entity as an attribute (which it does
    * automatically), that is, you should probably *NEVER* write a call to
    * this method explicitly.
    */
   protected abstract void removeBase(Entity base);

   ///////////////////////////////////////////////////////////////////////
   //
   //                     Entity Access
   //
   ///////////////////////////////////////////////////////////////////////

   /**
    * Request the Entity's name.
    */
   public abstract String getName() throws DataManagerException;

   /**
    * Request the Entity's value.
    */
   public abstract EntityValue getValue() throws DataManagerException;

   /**
    * Request a change of the Entity's value.
    */
   public abstract void setValue(EntityValue value)
         throws DataManagerException;

   /**
    * Request a constrained set of all entities linked to the Entity.
    */
   public abstract Entity[] getAttributes(EntityConstraint constraint)
         throws DataManagerException;

   /**
    * Request a constrained set of all entities of which this Entity
    * is an attribute.
    */
   public abstract Entity[] getBases(EntityConstraint constraint)
         throws DataManagerException;

   ///////////////////////////////////////////////////////////////////////
   //
   //                 Entity Subscription
   //
   ///////////////////////////////////////////////////////////////////////

   /**
    * Subscribe to the Entity for DataManagerEvents satisfying the
    * event constraint.
    */
   public abstract void subscribe(EventHandler listener,
         EventConstraint heard) throws DataManagerException;

   /**
    * Remove all event subscriptions the Simpleton has requested
    * from this Entity.
    */
   public abstract void unsubscribe(EventHandler handler)
         throws DataManagerException;

   public abstract String toString();
   }














