package org.datamanager.tools;

import java.io.File;
import java.io.FileFilter;
import java.io.IOException;
import java.io.*;
/**
 * A class to make compiling easier in the absence of the depends option.
 * The user can delete every classfile in all subfolders of a given
 * directory.  In addition, individual files or entire directories can
 * specified to be compiled.  Errors in compilation will be displayed on
 * the console screen.  
 * <br>
 * NOTE: This class is dependent upon Runtime.exec() being implemented on
 *       your system.  In addition, javac must be contained within your
 *       classpath.
 *
 * @author Gordon Murphy
 * @version .90
 */

public class Compile {
	private String classpath;
	public Compile(String[] classpath) {
		setClasspath(classpath);
		}
	public static final String[] CLASSPATHS = {".",
					"./startup/simpletons",
					"./startup/userinterface",
					"./tools/mail.jar",
					"./tools/pop3.jar",
					"./tools/Jama.jar"};

	public static void main(String[] parameters) throws IOException {
		File orgRoot = new File("org");
		File startupRoot = new File("startup");
		File[] removeDirectories = {orgRoot, startupRoot};
		File[] compileDirectories = {new File("org" + File.separator + 
								"datamanager" + File.separator +
								"userinterface" + File.separator +
								"cerulean" + File.separator +
								"Cerulean.java"),
							new File("startup" + File.separator +
								"simpletons"),
							new File("startup" + File.separator +
								"userinterface"),
							new File("org" + File.separator + 
								"datamanager" + File.separator +
								"kernel" + File.separator +
								"DataManager.java"),
							new File("org" + File.separator + 
								"datamanager" + File.separator +
								"userinterface" + File.separator +
								"cerulean" + File.separator +
								"shapes")
							};
		Compile compile = new Compile(CLASSPATHS);
		if (parameters.length > 0) {
			if (parameters[0].equals("-remove")) {
				compile.removeDirectories(removeDirectories);
				}
			else if (parameters[0].equals("-compile")) {
				compile.compile(compileDirectories);
				}
			}
		else {
			compile.removeDirectories(removeDirectories);
			compile.compile(compileDirectories);
			}
		}
	public void setClasspath(String[] paths) {
		String pathSeparator = System.getProperty("path.separator");
		String path = "";
		for (int i = 0; i <= paths.length - 1; i++) {
			path += paths[i];
			if (i < paths.length - 1) {
				path += pathSeparator;
				}
			}
		System.out.println(path);
		System.setProperty("java.class.path",path);
		classpath = path;
		}
	public void removeDirectories(File[] directories) throws IOException {
		for (int i = 0; i <= directories.length - 1; i++) {
			File directory = directories[i];
			if (directory.exists()) {
				removeAllClassFiles(directory);
				}
			}
		}
	public void compile(File[] files) throws IOException {
		for (int i = 0; i <= files.length - 1; i++) {	
			if (files[i].exists()) {
				String path = files[i].getAbsolutePath();
				if (files[i].isDirectory()) {	
					path += File.separator + "*.java";
					}
				exec("javac " + "-classpath " + classpath + " " + path);
				}
			}
		}
	protected void removeAllClassFiles(File directory) {
		System.out.println("opening: " + directory.getName());
		File[] subFiles = directory.listFiles();
		for (int i = 0; i < subFiles.length; i++) {
			if (subFiles[i].isDirectory()) {
				removeAllClassFiles(subFiles[i]);
				}
			}
		File[] classFiles = directory.listFiles(new ClassFileFilter());
		for (int i = 0; i < classFiles.length; i++) {
			classFiles[i].delete();
			System.out.println("deleting: "+classFiles[i].getName());
			}
		}


    protected void exec(String command) throws IOException {
		System.err.println("Executing: "+command);
		
		Runtime runtime = Runtime.getRuntime();
		Process p = runtime.exec(command);
		BufferedReader reader;
		reader = new BufferedReader(
			new InputStreamReader(
				p.getErrorStream()
				)
			);
		boolean isReading = true;
		String errorMessage = null;
		while (isReading) {
			try {
				errorMessage = reader.readLine();
				if (errorMessage != null) {
					System.out.println(errorMessage);
					}

				isReading = reader.ready();
				//System.out.println(1);
				}
			catch (IOException e) {
				isReading = false;
				}
			}
		reader.close();	
		try {
			int exitCode = p.waitFor();
			if(exitCode != 0) {
				System.err.println("Command " + command + " exited with code " + exitCode);
				System.exit(1);
				}
			}
		catch(InterruptedException ex) {
			System.err.println("Command " + command + " was interrupted : " + ex);
			System.exit(1);
			}
		finally {
			reader.close();
			}
	      }
/**
 * ClassFileFilter filters out all files except for those ending in .class
 */
private class ClassFileFilter implements FileFilter {
	public boolean accept(File file) {
		boolean isClassFile = false;
		boolean isAccepted = false;
		String fileName = file.getName();
		if (fileName.lastIndexOf(".class") != -1) {
			isClassFile = true;
			}
		if (file.isFile() && isClassFile) {
			isAccepted = true;
			}
		if ((fileName.indexOf("Compile") >=0) ||
			 (fileName.indexOf("ClassFileFilter") >= 0)) {
			isAccepted = false;
			}
		return isAccepted;
		}
	}
	}
