/*
class org.datamanager.userinterface.cerulean.model.InterfaceClusterManager
KnownSpace: Cerulean

Authors:
~~~~~~~~
Gordon Murphy, gpmurphy@indiana.edu
Bryan Dawson, bdawson@indiana.edu
*/

package org.datamanager.userinterface.cerulean.model;

import org.datamanager.userinterface.cerulean.controller.cluster.InterfaceClusterManagerClusterListener;
import org.datamanager.userinterface.cerulean.model.InterfaceCluster;
import org.datamanager.userinterface.cerulean.model.event.ClusterListener;
import org.datamanager.userinterface.cerulean.model.data.storage.ClusterList;
import java.util.Vector;
import java.io.Serializable;
/**
 * Every Cluster registers with InterfaceClusterManager upon inception.  Classes that 
 * wish to listen for cluster events for every cluster should do it through
 * this class.  A complete list of all clusters can also be had from here.
 *
 * @see ClusterListener
 * @see InterfaceClusterManagerClusterListener
 * @see InterfaceCluster
 * @see ClusterList
 *
 * @version 1.0
 * @author Gordon Murphy, gpmurphy@indiana.edu
 * @author Bryan Dawson, bdawson@indiana.edu 
 */

public class InterfaceClusterManager implements Serializable {
	private static InterfaceClusterManager clusterManager;
	private Vector listeners;
	private ClusterList clusters;

////////////////////////////////////////////////////////////////////////////////
/**
 * A private constructor.  Only one InterfaceClusterManager exists per session.  To
 * retrieve an instance of InterfaceClusterManager, call getInstance()
 */
	private InterfaceClusterManager() {

		//create the listeners and the vectors
		listeners = new Vector(10);
		clusters = new ClusterList();

		// a new clusterListener that can act as a central hub for
		// dispatching events to the knownspace event pool
		InterfaceClusterManagerClusterListener clusterListener;
		clusterListener = new InterfaceClusterManagerClusterListener(this);

		//add a clusterListener to all clusters
		addClusterListener(clusterListener);
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Returns an instance of InterfaceClusterManager
 * @return an instance of InterfaceClusterManager
 */
	public static InterfaceClusterManager getInstance() {
		if (clusterManager == null) {
			clusterManager = new InterfaceClusterManager();
			}
		return clusterManager;
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Registers clusters with the InterfaceClusterManager.  Also adds a cluster listener 
 * to monitor the events of the cluster
 * @param cluster the cluster to register
 */
	public void registerCluster(InterfaceCluster cluster) {
		clusters.addCluster(cluster);
		for(int i = 0; i <= listeners.size()-1; i++) {
			ClusterListener listener = (ClusterListener)listeners.elementAt(i);
			cluster.addClusterListener(listener);
			}
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Unregisters clusters so that a call to addClusterListener will add a cluster
 * listener to all clusters.  Also removes all cluster listener added to a
 * cluster by the manager
 * @param cluster the cluster to unregister
 */
	public void unregisterCluster(InterfaceCluster cluster) {
		clusters.removeCluster(cluster);
		for(int i = 0; i <= listeners.size()-1; i++) {
			ClusterListener listener = (ClusterListener)listeners.elementAt(i);
			cluster.removeClusterListener(listener);
			}
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Adds a cluster listener to all registered clusters
 * @param listener the listener to add to a cluster
 */
	public void addClusterListener(ClusterListener listener) {
		listeners.add(listener);
		for(int i = 0; i <= clusters.size()-1; i++) {
			InterfaceCluster cluster = (InterfaceCluster)clusters.elementAt(i);
			cluster.addClusterListener(listener);
			}
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Removes a cluster listener from all registered clusters
 * @param listener the listener to remove from a cluster
 */
	public void removeClusterListener(ClusterListener listener) {
		listeners.remove(listener);
		for(int i = 0; i <= clusters.size()-1; i++) {
			InterfaceCluster cluster = (InterfaceCluster)clusters.elementAt(i);
			cluster.removeClusterListener(listener);
			}
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Returns all registered clusters
 * @return a ClusterList of all registered clusters
 */
	public ClusterList getClusters() {
		return clusters;
		}	

////////////////////////////////////////////////////////////////////////////////
/**
 * Returns all registered clusters
 * @return a ClusterList of all registered clusters
 */
	public ClusterList getClusterList() {
		return clusters;
		}
////////////////////////////////////////////////////////////////////////////////
/**
 * Returns all clusters containing the specified name
 * @param the name to search for
 * @returns the first cluster found containing the name
 */
	public InterfaceCluster findCluster(String name) {
		ClusterList list = getClusters();
		InterfaceCluster cluster = null;
		for (int i = 0; i < list.size(); i++) {
			cluster = list.elementAt(i);
			if (cluster.getName().equals(name)) {
				break;
				}
			}
		return cluster;
		}
////////////////////////////////////////////////////////////////////////////////
/**
 * Returns the Search cluster
 * @returns the search cluster
 */
	public InterfaceCluster getSearchCluster() {
		return findCluster("search");
		}	
////////////////////////////////////////////////////////////////////////////////
/**
 * Returns the New Entities cluster
 * @returns the new entities cluster
 */
	public InterfaceCluster getNewEntitiesCluster() {
		return findCluster("new");
		}
////////////////////////////////////////////////////////////////////////////////
/**
 * Returns the Recommended cluster
 * @returns the recommended cluster
 */
	public InterfaceCluster getRecommendedCluster() {
		return findCluster("recommended");
		}
	}