/*
class org.datamanager.userinterface.cerulean.model.InterfaceClusterPeerTable
KnownSpace: Cerulean

Authors:
~~~~~~~~
Gordon Murphy, gpmurphy@indiana.edu
Bryan Dawson, bdawson@indiana.edu
*/

package org.datamanager.userinterface.cerulean.model;

import org.datamanager.userinterface.cerulean.view.*;
import org.datamanager.userinterface.cerulean.view.component.*;
import org.datamanager.userinterface.cerulean.model.*;
import org.datamanager.userinterface.cerulean.model.event.*;
import org.datamanager.userinterface.cerulean.model.data.transfer.*;
import java.util.Hashtable;
import java.util.Enumeration;
import java.io.Serializable;

/**
 * A class that maintains a list of all InterfaceClusters and their corresponding
 * InterfaceClusterComponents.  To register a cluster with a peer, simply call 
 * registerPeer(cluster, clusterComponent).  Then to get the InterfaceClusterComponent
 * when only a InterfaceCluster is known, call getPeer() and pass it the InterfaceCluster.
 *
 * example of registering a peer:
 * InterfaceCluster cluster = new InterfaceCluster() //create an empty cluster
 * InterfaceClusterComponent component = new InterfaceClusterComponent(cluster);
 * InterfaceClusterPeerTable peerTable = InterfaceClusterPeerTable.getInstance();
 * peerTable.registerPeer(cluster, component);
 * 
 * Then to get the InterfaceCluster simply call:
 * InterfaceClusterComponent clusterComponent = peerTable.getPeer(cluster);
 *
 * @see InterfaceCluster
 * @see InterfaceClusterComponent
 *
 * @version 1.0
 * @author Gordon Murphy, gpmurphy@indiana.edu
 * @author Bryan Dawson, bdawson@indiana.edu 
 */

public class InterfaceClusterPeerTable implements Serializable {
	private Hashtable peerTable;
	private static InterfaceClusterPeerTable defaultPeerTable = new InterfaceClusterPeerTable();
////////////////////////////////////////////////////////////////////////////////
/**
 * A private constructor.  To get the peerTable call 
 * InterfaceClusterPeerTable.getInstance();
 */
	private InterfaceClusterPeerTable() {
		peerTable = new Hashtable(100, .65f);
		}
////////////////////////////////////////////////////////////////////////////////
/**
 * Retrieves an instance of the InterfaceClusterPeerTable.
 * @return the default InterfaceClusterPeerTable
 */
	public synchronized static InterfaceClusterPeerTable getInstance() {
		return defaultPeerTable;
		}
////////////////////////////////////////////////////////////////////////////////
/**
 * Registers a cluster with its associated component
 * @param cluster the cluster to register
 * @param clusterComponent the component to register
 */
	public synchronized void registerPeer(InterfaceCluster cluster,
					 InterfaceClusterComponent clusterComponent) {
		peerTable.put(cluster, clusterComponent);
		}
////////////////////////////////////////////////////////////////////////////////
/**
 * Unregisters a cluster with its associated component
 * @param cluster the cluster to unregister
 * @param clusterComponent the component to unregister
 */
	public synchronized void unregisterPeer(InterfaceCluster cluster,
					 InterfaceClusterComponent clusterComponent) {
		peerTable.remove(cluster);
		}
////////////////////////////////////////////////////////////////////////////////
/**
 * Retrieves the InterfaceClusterComponent associated with the specified InterfaceCluster
 * @param cluster the cluster to retrieve the peer of
 * @return the component associated with the cluster
 */
	public synchronized InterfaceClusterComponent getPeer(InterfaceCluster cluster) {
		return (InterfaceClusterComponent)peerTable.get(cluster);
		}
////////////////////////////////////////////////////////////////////////////////
/**
 * Retrieves all InterfaceClusterComponent associated with the InterfaceClusters
 * @return an array of InterfaceClusterComponents
 */
	public synchronized InterfaceClusterComponent[] getAllPeers() {
		InterfaceClusterComponent[] components;
		components = new InterfaceClusterComponent[peerTable.size()];
		Enumeration enumeration = peerTable.elements();
		int i = 0;
		while(enumeration.hasMoreElements()) {
			components[i] = (InterfaceClusterComponent)enumeration.nextElement();
			i++;
			}
		return components;
		}
	}