/*
class org.datamanager.userinterface.cerulean.model.InterfaceEntity
KnownSpace: Cerulean

Authors:
~~~~~~~~
Gordon Murphy, gpmurphy@indiana.edu
Bryan Dawson, bdawson@indiana.edu
*/

package org.datamanager.userinterface.cerulean.model;

import org.datamanager.userinterface.cerulean.view.*;
import org.datamanager.userinterface.cerulean.model.*;
import org.datamanager.tools.*;
import org.datamanager.userinterface.cerulean.model.kernel.*;
import org.datamanager.userinterface.cerulean.model.event.*;
import org.datamanager.userinterface.cerulean.model.data.transfer.*;
import org.datamanager.userinterface.cerulean.model.data.storage.*;
import org.datamanager.kernel.Entity;
import org.datamanager.kernel.*;
import org.datamanager.constraint.*;
import org.datamanager.passiveentityvalue.*;
import org.datamanager.exception.DataManagerException;
import java.io.Serializable;
/**
 * This class is somewhat of a wrapper class around the Entity class of the
 * kernel.  It fires EntityEvent
 */
public class InterfaceEntity implements Serializable {
	private InterfaceEntityEventManager eventManager;
	private Entity entity;
	private int x;
	private int y;
	private int z;
	private InterfaceCluster parentCluster, oldCluster;

////////////////////////////////////////////////////////////////////////////////
/**
 * Create a new InterfaceEntity from the given Entity
 * @param entity an entity the InterfaceEntity will represent
 */
	public InterfaceEntity(Entity entity) {
		this.entity = entity;
		this.parentCluster = null;
		this.oldCluster = parentCluster;
		createName();
		setCreated(this);
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Create a null instance of a InterfaceEntity
 */
	public InterfaceEntity() {
		this("");
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Create a null instance of a InterfaceEntity
 */
	public InterfaceEntity(String name) {
		this(KernelProxy.getInstance().createEntity(name));
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Returns the value of the InterfaceEntity.
 * @return the EntityValue that this entity represents
 */
	public EntityValue getValue() {
		return KernelProxy.getInstance().getEntityValue(getEntity());
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Set the value of the entity to the specified double value.
 */
	public void setValue(double value) {
		DoubleEntityValue entityValue = new DoubleEntityValue(value);
		setValue(entityValue);
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Set the value of the entity to the specified String value.
 */
	public void setValue(String value) {
		StringEntityValue entityValue = new StringEntityValue(value);
		setValue(entityValue);
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Set the value of the entity to the specified integer value.
 */
	public void setValue(int value) {
		IntegerEntityValue entityValue = new IntegerEntityValue(value);
		setValue(entityValue);
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Set the value of the entity to the specified boolean value.
 */
	public void setValue(boolean value) {
		BooleanEntityValue entityValue = new BooleanEntityValue(value);
		setValue(entityValue);
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Set the value of the entity to the specified EntityValue.
 * @param value an EntityValue
 */
	public void setValue(EntityValue value) {
		KernelProxy.getInstance().setEntityValue(getEntity(), value);
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Attaches the specified InterfaceAttribute to this entity.
 * @param attribute the attribute to attach.
 */
	public void attachAttribute(InterfaceAttribute attribute) {
		KernelProxy kernel = KernelProxy.getInstance();		
		kernel.attachEntity(getEntity(), attribute.getEntity());
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Detaches the specified InterfaceAttribute from this entity.
 * @param attribute the attribute to detach.
 */
	public void detachAttribute(InterfaceAttribute attribute) {
		KernelProxy kernel = KernelProxy.getInstance();		
		kernel.detachEntity(getEntity(), attribute.getEntity());
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Returns the first attribute with attributeName found that is
 * attached to the entity.
 * @param attributeName the name to match
 * @return an InterfaceAttribute matching attributeName
 */
	public InterfaceAttribute getAttribute(String attributeName) {
		return this.getAttribute(new NameEqualsConstraint(attributeName));
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Returns the first attribute matching constraint found that is
 * attached to the entity.
 * @param constraint the constraint to match
 * @return an InterfaceAttribute matching constraint
 */
	public InterfaceAttribute getAttribute(EntityConstraint constraint) {
		InterfaceAttributeList list = getAttributes(constraint);
		InterfaceAttribute attribute = null;
		if (list.size() >=1) {
			attribute = (InterfaceAttribute)list.elementAt(0);
			}
		return attribute;
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Returns all attributes attached to the entity.
 * @return all attached attributes
 */
	public InterfaceAttributeList getAttributes() {
		return this.getAttributes(EntityConstraint.NONE);
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Returns all attributes with attributeName attached to the entity.
 * @param attributeName the name to match
 * @return all attached attributes matching the specified name
 */
	public InterfaceAttributeList getAttributes(String attributeName) {
		return this.getAttributes(new NameEqualsConstraint(attributeName));
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Returns all attributes matching constraint attached to the entity.
 * @param constraint the constraint to match
 * @return all attached attributes matching constraint
 */
	public InterfaceAttributeList getAttributes(EntityConstraint constraint) {
		KernelProxy kernel = KernelProxy.getInstance();
		Entity[] attributes = kernel.getAttributes(getEntity(),constraint);
		InterfaceAttributeList list = new InterfaceAttributeList(attributes);
		return list;
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Returns whether or not an attributeis attached.
 * @return the bases this entity is attached to.
 */
	public boolean isAttributeAttached() {
		return this.isAttributeAttached(EntityConstraint.NONE);
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Returns whether or not an attribute with the name specified by attributeName
 * is attached.
 * @param attributeName the name of an attached attribute
 * @return the bases this entity is attached to.
 */
	public boolean isAttributeAttached(String attributeName) {
		return this.isAttributeAttached(new NameEqualsConstraint(attributeName));
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Returns whether an attribute matching the given constraint are attached.
 * @param constraint a constraint that attached attributes must match.
 * @return true if is attached, false otherwise.
 */
	public boolean isAttributeAttached(EntityConstraint constraint) {
		return !(getAttribute(constraint) == null);
		}	

////////////////////////////////////////////////////////////////////////////////
/**
 * Returns all the bases matching baseName that this entity is attached to.
 * @param baseName a string specifying the name of the base this entity is attached to
 * @return the bases this entity is attached to.
 */
	public InterfaceEntityList getBases(String baseName) {
		return this.getBases(new NameEqualsConstraint(baseName));
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Returns all the bases which this entity is attached to.
 * @return the bases this entity is attached to.
 */
	public InterfaceEntityList getBases(EntityConstraint constraint) {
		KernelProxy kernel = KernelProxy.getInstance();
		Entity[] bases = kernel.getAttributes(getEntity(),constraint);
		InterfaceEntityList list = new InterfaceEntityList(bases);
		return list;
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Attach an attribute specifying the entity's name.
 */
	protected void createName() {
		KernelProxy kernel = KernelProxy.getInstance();
		if (!isAttributeAttached("InterfaceName")) {
			InterfaceAttribute nameAttribute = new InterfaceAttribute("InterfaceName");
			String name = kernel.getEntityName(getEntity());
			nameAttribute.setValue(name);
			attachAttribute(nameAttribute);
			}
		}
////////////////////////////////////////////////////////////////////////////////
/**
 * Sets the entity's name.
 * @param entityName the new name of the entity
 */
	public void setEntityName(String entityName) {
		InterfaceAttribute attribute = getAttribute("InterfaceName");
		attribute.setValue(entityName);
		setModified(this);
		}
////////////////////////////////////////////////////////////////////////////////
/**
 * Gets the entity's name.
 * @param entityName the new name of the entity
 * @return the entity's name
 */
	public String getEntityName() {		
		InterfaceAttribute attribute = getAttribute("InterfaceName");
		EntityValue entityValue = attribute.getValue();
		String name = entityValue.toString();
		return name;
		}
////////////////////////////////////////////////////////////////////////////////
/**
 * Get the name of the Entity
 * @return the entity's name
 */
	public String getName() {
		return getEntityName();
		}
////////////////////////////////////////////////////////////////////////////////
/**
 * Returns the Entity which the InterfaceEntity represents
 * @return the represented entity
 */
	protected Entity getEntity() {
		return entity;
		}

////////////////////////////////////////////////////////////////////////////////
	private void setCoordinates(int x, int y, int z) {
		this.x = x;
		this.y = y;
		this.z = z;
		}

////////////////////////////////////////////////////////////////////////////////
	private int getX() {
		return x;
		}

////////////////////////////////////////////////////////////////////////////////
	private int getY() {
		return y;
		}

////////////////////////////////////////////////////////////////////////////////
	private int getZ() {
		return z;
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Returns the last cluster this entity belonged to.
 * @return the last cluster this entity belonged to.
 */
	public InterfaceCluster getOldCluster() {
		return oldCluster;
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Returns the current cluster this entity is part of.
 * @return this entity's parent cluster.
 */
	public InterfaceCluster getParentCluster() {
		return parentCluster;
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Sets the current cluster this entity is a part of.
 * @param parentCluster this entity's parent cluster.
 */
	public void setParentCluster(InterfaceCluster parentCluster) {
		this.oldCluster = this.parentCluster;
		this.parentCluster = parentCluster;
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Removes the interface entity from all of knownspace
 */
	public void remove() {
		if (getParentCluster() != null) {
			getParentCluster().removeEntity(this);
			}
		InterfaceAttributeList attributes = getAttributes();
		for (int i = 0; i < attributes.size(); i++) {
			attributes.elementAt(i).remove();
			}
		KernelProxy.getInstance().removeEntity(getEntity());
		}
////////////////////////////////////////////////////////////////////////////////
/**
 * Sets the entity as having been created.
 * @param source the source of the creation
 */
	public void setCreated(Object source) {
		getEntityEventManager().fireEntityCreated(source, this);
		}
////////////////////////////////////////////////////////////////////////////////
/**
 * Sets the entity as having been selected.
 * @param source the source of the selection
 */
	public void setSelected(Object source) {
		getEntityEventManager().fireEntitySelected(source, this);
		}
////////////////////////////////////////////////////////////////////////////////
/**
 * Sets the entity as having been unselected.
 * @param source the source of the selection
 */
	public void setUnselected(Object source) {
		getEntityEventManager().fireEntityUnselected(source, this);
		}
////////////////////////////////////////////////////////////////////////////////
/**
 * Sets the entity as having been opened
 * @param source the source of the opening
 */
	public void setOpened(Object source) {
		getEntityEventManager().fireEntityOpened(source, this);
		}
////////////////////////////////////////////////////////////////////////////////
/**
 * Sets the entity as having been attached to another entity.
 * @param source the source of the attachment
 */
	public void setAttached(Object source, InterfaceEntity attachedEntity) {
		KernelProxy.getInstance().attachEntity(getEntity(), attachedEntity.getEntity());
		getEntityEventManager().fireEntityAttached(source, this, attachedEntity);
		}
////////////////////////////////////////////////////////////////////////////////
/**
 * Sets the entity as having been detached from another entity
 * @param source the source of the detachment
 */
	public void setDetached(Object source, InterfaceEntity detachedEntity) {
		KernelProxy.getInstance().detachEntity(getEntity(), detachedEntity.getEntity());	
		getEntityEventManager().fireEntityDetached(source, this, detachedEntity);	
		}
////////////////////////////////////////////////////////////////////////////////
/**
 * Sets the entity as having been moved.
 * @param source the source of the movement
 * @param x the x coordinate of the movement
 * @param y the y coordinate of the movement
 */
	public void setMoved(Object source, int x, int y) {
		this.setMoved(source, x, y, 0);
		}
////////////////////////////////////////////////////////////////////////////////
/**
 * Sets the entity as having been moved
 * @param source the source of the movement
 * @param x the x coordinate of the movement
 * @param y the y coordinate of the movement
 * @param z the z coordinate of the movement
 */
	public void setMoved(Object source, int x, int y, int z) {
		setCoordinates(x,y,z);
		getEntityEventManager().fireEntityMoved(source, this, x, y, z);
		}

////////////////////////////////////////////////////////////////////////////////
	public void setRelativeMoved(Object source, int x, int y) {
		this.setRelativeMoved(source,x,y,0);
		}

////////////////////////////////////////////////////////////////////////////////
	public void setRelativeMoved(Object source, int x, int y, int z) {
		this.setMoved(source, getX()+x, getY()+y, getZ()+z);
		}
////////////////////////////////////////////////////////////////////////////////
/**
 * Sets the entity as having been modified.
 * @param source the source of the selection
 */
	public void setModified(Object source) {
		eventManager.fireEntityModified(source, this);
		}

////////////////////////////////////////////////////////////////////////////////
	public InterfaceEntityEventManager getEntityEventManager() {
		if (eventManager == null) {
			eventManager = new InterfaceEntityEventManager();
			InterfaceEntityManager.getInstance().registerEntity(this);
			}
		return eventManager;
		}

////////////////////////////////////////////////////////////////////////////////
	public void addEntityListener(EntityListener listener) {
		getEntityEventManager().addEntityListener(listener);
		}

////////////////////////////////////////////////////////////////////////////////
	public void removeEntityListener(EntityListener listener) {
		getEntityEventManager().removeEntityListener(listener);
		}	

////////////////////////////////////////////////////////////////////////////////
	public String toString() {
		return "InterfaceEntity: " + getEntityName();
		}
	}