/*
class org.datamanager.userinterface.cerulean.model.InterfaceEntityManager
KnownSpace: Cerulean

Authors:
~~~~~~~~
Gordon Murphy, gpmurphy@indiana.edu
Bryan Dawson, bdawson@indiana.edu
*/
////////////////////////////////////////////////////////////////////////////////

package org.datamanager.userinterface.cerulean.model;

import java.util.HashMap;
import org.datamanager.userinterface.cerulean.view.component.InterfaceEntityComponent;
import org.datamanager.userinterface.cerulean.model.InterfaceEntity;

////////////////////////////////////////////////////////////////////////////////
/**
 * A class that maintains a list of all InterfaceEntitys and their corresponding
 * InterfaceEntityComponents.  To register a entity with a peer, simply call 
 * registerPeer(entity, entityComponent).  Then to get the InterfaceEntityComponent
 * when only a InterfaceEntity is known, call getPeer() and pass it the InterfaceEntity.
 *
 * example of registering a peer:
 * InterfaceEntity entity = new InterfaceEntity() //create an empty Entity
 * InterfaceEntityComponent component = new InterfaceEntityComponent(entity);
 * InterfaceEntityPeerTable peerTable = InterfaceEntityPeerTable.getInstance();
 * peerTable.registerPeer(entity, component);
 * 
 * Then to get the InterfaceEntity simply call:
 * InterfaceEntityComponent entityComponent = peerTable.getPeer(entity);
 *
 * @see InterfaceEntity
 * @see InterfaceEntityComponent
 *
 * @version 1.0
 * @author Gordon Murphy, gpmurphy@indiana.edu
 * @author Bryan Dawson, bdawson@indiana.edu 
 */

public class InterfaceEntityPeerTable {
	private HashMap peerTable;
	private static InterfaceEntityPeerTable defaultPeerTable = new InterfaceEntityPeerTable();

////////////////////////////////////////////////////////////////////////////////
/**
 *
 */
	private InterfaceEntityPeerTable() {
		peerTable = new HashMap(1000, .55f);
		}
////////////////////////////////////////////////////////////////////////////////
/**
 * Retrieves an instance of the InterfaceEntityPeerTable.
 * @return the default InterfaceEntityPeerTable
 */
	public synchronized static InterfaceEntityPeerTable getInstance() {
		return defaultPeerTable;
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Registers an entity with its associated component
 * @param entity the entity to register
 * @param entityComponent the component to register
 */
	public synchronized void registerPeer(InterfaceEntity entity, InterfaceEntityComponent entityComponent) {
		peerTable.put(entity, entityComponent);
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Unregisters an entity with its associated component
 * @param entity the entity to unregister
 * @param entityComponent the component to unregister
 */
	public synchronized void unregisterPeer(InterfaceEntity entity) {
		peerTable.remove(entity);
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Retrieves the InterfaceEntityComponent associated with the specified
 * InterfaceEntity.
 * @param entity the entity to retrieve the peer of
 * @return the component associated with the entity
 */
	public synchronized InterfaceEntityComponent getPeer(InterfaceEntity entity) {
		return (InterfaceEntityComponent)peerTable.get(entity);
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Returns true if the table contains the component.
 * @param entity the entityComponent checking to see if it is contained
 * @return true if found, false otherwise
 */
	public synchronized boolean contains(InterfaceEntityComponent entity) {
		return peerTable.containsValue(entity);
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Returns true if the table contains the entity.
 * @param entity the entity checking to see if it is contained
 * @return true if found, false otherwise
 */
	public synchronized boolean containsEntity(InterfaceEntity entity) {
		return peerTable.containsKey(entity);
		}
	}