/*
class org.datamanager.userinterface.cerulean.model.InterfaceCluster
KnownSpace: Cerulean

Authors:
~~~~~~~~
Gordon Murphy, gpmurphy@indiana.edu
Bryan Dawson, bdawson@indiana.edu
*/
////////////////////////////////////////////////////////////////////////////////

package org.datamanager.userinterface.cerulean.model;

import java.io.Serializable;
import java.lang.Object;
import org.datamanager.userinterface.cerulean.controller.cluster.StageClusterListener;
import org.datamanager.userinterface.cerulean.model.data.storage.ClusterList;
import org.datamanager.userinterface.cerulean.model.data.storage.InterfaceEntityList;
import org.datamanager.userinterface.cerulean.model.event.StageEventManager;
import org.datamanager.userinterface.cerulean.model.event.StageListener;

////////////////////////////////////////////////////////////////////////////////

/**
 * The Stage is a place where entities can be displayed.  Entities may be added
 * to the stage and removed from the stage.  When an entity is added, it is
 * "displayed".  Note this class itself has no visual representation.  Objects
 * may register to listen for StageEvents, which are fired when certain actions
 * happen.
 *
 * @see StageEventManager
 * @see StageListener
 * @see StageComponent
 * @see StageClusterListener
 *
 * @author Gordon Murphy gpmurphy@indiana.edu
 * @author Bryan Dawson bdawson@indiana.edu
 * @version 1.0
 */

public class Stage implements Serializable {
	private InterfaceClusterManager clusterManager;
	private ClusterList clusterList;
	private InterfaceEntityList entityList;
	private StageEventManager eventManager;
////////////////////////////////////////////////////////////////////////////////
/**
 * Creates a new Stage with with no entities or clusters inside of it.
 */
	public Stage() {
		clusterList = new ClusterList();
		entityList = new InterfaceEntityList();
		eventManager = new StageEventManager();
		clusterManager = InterfaceClusterManager.getInstance();
		clusterManager.addClusterListener(new StageClusterListener(this));
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Retrieve a list of all clusters currently displayed in the stage.
 * @return the displayed clusters
 */ 
	public ClusterList getClusterList() {
		return clusterList;
		}
////////////////////////////////////////////////////////////////////////////////
/**
 * Retrieve a list of all entities currently displayed in the stage.
 * @return the displayed entities
 */
	public InterfaceEntityList getInterfaceEntityList() {
		return entityList;
		}
////////////////////////////////////////////////////////////////////////////////
/**
 * Add a cluster to be displayed in the stage.
 * @param cluster a cluster to add to the stage
 */
	public void addCluster(InterfaceCluster cluster) {
		clusterList.addCluster(cluster);
		InterfaceEntityList entities = cluster.getInterfaceEntityList();
		addEntities(entities);
		}
////////////////////////////////////////////////////////////////////////////////
/**
 * Remove a cluster from being displayed in the stage
 * @param cluster a cluster to remove from the stage
 */
	public void removeCluster(InterfaceCluster cluster) {
		clusterList.removeCluster(cluster);
		InterfaceEntityList entities = cluster.getInterfaceEntityList();
		removeEntities(entities);
		}
////////////////////////////////////////////////////////////////////////////////
/**
 * Add an entity to be displayed in the stage.
 * @param entity an entity to add to the stage
 */
	public void addEntity(InterfaceEntity entity) {
		InterfaceEntityList list = new InterfaceEntityList();
		list.addEntity(entity);
		this.addEntities(list);
		}
////////////////////////////////////////////////////////////////////////////////
/**
 * Add a list of entities to be displayed in the stage.
 * @param entities entities to display in the stage
 */
	public void addEntities(InterfaceEntityList entities) {		
		entityList.addEntities(entities);
		eventManager.fireEntitiesDisplayed(this, entities);
		}
////////////////////////////////////////////////////////////////////////////////
/**
 * Remove an entity from being displayed in the stage
 * @param entity an entity to remove from the stage
 */
	public void removeEntity(InterfaceEntity entity) {
		InterfaceEntityList list = new InterfaceEntityList();
		list.addEntity(entity);
		this.removeEntities(list);
		}
////////////////////////////////////////////////////////////////////////////////
/**
 * Remove entities from being displayed in the stage
 * @param entities entities to remove from the stage
 */
	public void removeEntities(InterfaceEntityList entities) {
		entityList.removeEntities(entities);
		eventManager.fireEntitiesUndisplayed(this, entities);
		}
////////////////////////////////////////////////////////////////////////////////
/**
 * Register to listen for stage events.
 * @param listener the listener that will handle events
 */
	public void addStageListener(StageListener listener) {
		eventManager.addStageListener(listener);
		}
////////////////////////////////////////////////////////////////////////////////
/**
 * Register to listen for stage events.
 * @param listener the listener that will handle events
 */
	public void removeStageListener(StageListener listener) {
		eventManager.removeStageListener(listener);
		}
	}