package org.datamanager.userinterface.cerulean.model.event;
import org.datamanager.userinterface.cerulean.model.*;
import org.datamanager.userinterface.cerulean.model.data.storage.InterfaceEntityList;
import org.datamanager.userinterface.cerulean.model.data.storage.ClusterList;
import java.util.Hashtable;

/**
 * An event that marks the reclustering of entities from one 
 * cluster to another.  ClusterListeners may retrieve a list
 * of the old clusters that entities have been moved from, in
 * addition to finding the reverse.
 * <p>
 * Here is an example of finding out whether entities were reclustered
 * from a cluster:<br />
 * if (e.getOldClusters().contains(cluster)) {<br />
 *    InterfaceEntityList entities = e.getEntitiesReclusteredFrom(cluster);<br />
 *    }<br />
 * This will tell you both whether or not entities were reclustered from
 * that cluster, and if they were, they will tell you which entities were 
 * removed.</p>
 *
 * @see InterfaceCluster
 * @see InterfaceClusterManager
 * @see ClusterListener
 *
 * @author Gordon Murphy
 */
public class ClusterReclusteringEvent extends ClusterContentEvent {
	protected ClusterList oldClusters;
	protected InterfaceCluster newCluster;
	protected Hashtable entityClusterMap;
/**
 * Create a new event object.
 * @param source the source of the event
 * @param entities the entities that have been reclustered
 * @param oldClusters all the clusters that entities were removed from
 * @param entityClusterMap a hashtable containing a key-value pairing
 *  between clusters and an associated entityList of entities moved
 *  from that cluster.
 */
	public ClusterReclusteringEvent(Object source,
					 InterfaceEntityList entities,
					 ClusterList oldClusters,
					 InterfaceCluster newCluster,
					 Hashtable entityClusterMap) {
		super(source, newCluster, entities);
		this.oldClusters = oldClusters;
		this.newCluster = newCluster;
		this.entityClusterMap = entityClusterMap;
		}
/**
 * Retrieve all of the old clusters that entities were moved from.
 * @return a list of clusters
 */
	public ClusterList getOldClusters() {
		return oldClusters;
		}
/**
 * Retrieve the cluster the entities were moved to.
 * @return the cluster that contains the entities.
 */
	public InterfaceCluster getNewCluster() {
		return newCluster;
		}
/**
 * Retrieves the entities that were moved from cluster
 * @param cluster the cluster that contained the entities.
 * @return the entities formerly contained by cluster.
 */
	public InterfaceEntityList getEntitiesReclusteredFrom(InterfaceCluster cluster) {
		return (InterfaceEntityList)(entityClusterMap.get(cluster));
		}
	}