package org.datamanager.userinterface.cerulean.model.event;
import org.datamanager.userinterface.cerulean.view.*;
import org.datamanager.userinterface.cerulean.model.*;
import org.datamanager.userinterface.cerulean.model.event.*;
import org.datamanager.userinterface.cerulean.model.event.cluster.*;
import org.datamanager.userinterface.cerulean.model.data.transfer.*;
import org.datamanager.userinterface.cerulean.model.data.storage.InterfaceEntityList;
import org.datamanager.userinterface.cerulean.model.data.storage.ClusterList;
import org.datamanager.tools.Debug;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.awt.geom.*;
import java.util.Hashtable;
/**
 * Manages the events of clusters.  Keeps track of registered interested
 * ClusterListeners and fires events to them.
 *
 * @see InterfaceCluster
 * @see ClusterListener
 *
 * @author Gordon Murphy (gpmurphy@indiana.edu)
 */
public class InterfaceClusterEventManager {
	private EventListenerList clusterListenerList;
	private ClusterEvent clusterEvent;
/**
 * Creates a new event manager with no cluster listeners.
 */
	public InterfaceClusterEventManager() {
		clusterListenerList = new EventListenerList();
		}
/**
 * Registers a cluster listener to listen for events.
 * @param listener an interested listener
 */
	public void addClusterListener(ClusterListener listener) {
		clusterListenerList.add(listener.getClass(), listener);
		}
/**
 * Unregisters a cluster listener.
 * @param listener the listener to unregister.
 */
	public void removeClusterListener(ClusterListener listener) {
		clusterListenerList.remove(listener.getClass(), listener);
		}
/**
 * Fires a cluster added event.
 * @param source the originator of the event
 * @cluster the cluster that was added.
 */
	public synchronized void fireClusterAdded(Object source, InterfaceCluster cluster) {
		Object[] listeners = clusterListenerList.getListenerList();
		ClusterEvent clusterEvent = new ClusterEvent(source,cluster);
		for(int i = listeners.length - 2; i>=0; i-=2) {	
			((ClusterListener)listeners[i+1]).clusterAdded(clusterEvent);
			}
		}
/**
 * Fires a cluster removed event.
 * @param source the originator of the event.
 * @param cluster the cluster to be removed.
 */
	public synchronized void fireClusterRemoved(Object source, InterfaceCluster cluster) {
		Object[] listeners = clusterListenerList.getListenerList();
		ClusterEvent clusterEvent = new ClusterEvent(source,cluster);
		for(int i = listeners.length - 2; i>=0; i-=2) {	
			((ClusterListener)listeners[i+1]).clusterRemoved(clusterEvent);
			}
		}
/**
 * Fires a cluster selected event.
 * @param source the originator of the event.
 * @param cluster the selected cluster.
 */
	public synchronized void fireClusterSelected(Object source, InterfaceCluster cluster) {
		Object[] listeners = clusterListenerList.getListenerList();
		ClusterEvent clusterEvent = new ClusterEvent(source,cluster);
		for(int i = listeners.length - 2; i>=0; i-=2) {	
			((ClusterListener)listeners[i+1]).clusterSelected(clusterEvent);
			}
		}
/**
 * Fire a cluster unselected event.
 * @param source the originator of the event.
 * @param cluster the unselected cluster.
 */
	public synchronized void fireClusterUnselected(Object source, InterfaceCluster cluster) {
		Object[] listeners = clusterListenerList.getListenerList();
		ClusterEvent clusterEvent = new ClusterEvent(source,cluster);
		for(int i = listeners.length - 2; i>=0; i-=2) {	
			((ClusterListener)listeners[i+1]).clusterUnselected(clusterEvent);
			}
		}

	public synchronized void fireClusterModified(Object source, InterfaceCluster cluster,InterfaceEntity entity,boolean wasAdded) {
		Object[] listeners = clusterListenerList.getListenerList();
		ClusterModifiedEvent clusterEvent = new ClusterModifiedEvent(source,cluster,entity,wasAdded);
		for(int i = listeners.length - 2; i>=0; i-=2) {	
			((ClusterListener)listeners[i+1]).clusterModified(clusterEvent);
			}
		}
/**
 * Fires an entities added to cluster event.
 * @param source the originator of the event.
 * @param cluster the cluster entities were added to.
 * @param entities the entities added to the cluster.
 */
	public synchronized void fireEntitiesAddedToCluster(Object source, InterfaceCluster cluster,InterfaceEntityList entities) {
		Object[] listeners = clusterListenerList.getListenerList();
		ClusterContentEvent clusterEvent = new ClusterContentEvent(source,cluster,entities);
		for(int i = listeners.length - 2; i>=0; i-=2) {	
			((ClusterListener)listeners[i+1]).entitiesAddedToCluster(clusterEvent);
			}
		}
/**
 * Fires an entities removed from cluster event.
 * @param source the originator of the event.
 * @param cluster the cluster entities were removed from.
 * @param entities the entities removed from the cluster.
 */ 
	public synchronized void fireEntitiesRemovedFromCluster(Object source, InterfaceCluster cluster,InterfaceEntityList entities) {
		Object[] listeners = clusterListenerList.getListenerList();
		ClusterContentEvent clusterEvent = new ClusterContentEvent(source,cluster,entities);
		for(int i = listeners.length - 2; i>=0; i-=2) {	
			((ClusterListener)listeners[i+1]).entitiesRemovedFromCluster(clusterEvent);
			}
		}
/**
 * Fires an entities reclustered event.
 * @param source the originator of the event.
 * @param entities the entities reclustered.
 * @param oldClusters the clusters that used to contain the entities.
 * @param newCluster the new parent cluster of the entities.
 * @param entityClusterMap a mapping between the oldClusters and the entities
 *  they contained.
 */
	public synchronized void fireEntitiesReclustered(Object source,
						 InterfaceEntityList entities,
						 ClusterList oldClusters,
						 InterfaceCluster newCluster,
						 Hashtable entityClusterMap) {
		Object[] listeners = clusterListenerList.getListenerList();
		ClusterReclusteringEvent clusterEvent;
		clusterEvent = new ClusterReclusteringEvent(source,entities,oldClusters,newCluster, entityClusterMap);
		for(int i = listeners.length - 2; i>=0; i-=2) {	
			((ClusterListener)listeners[i+1]).entitiesReclustered(clusterEvent);
			}
		}
	}