package org.datamanager.userinterface.cerulean.model.kernel;

import javax.swing.SwingUtilities;
import org.datamanager.kernel.DataManagerWorker;
import org.datamanager.passiveentityvalue.BooleanEntityValue;

/**
 * An abstract class that can be used to interact with the kernel.
 * Essentially, a new InterfaceWorker is created.  Its contents must
 * be registered by the user, so that the thread is run.  The results
 * of the operation be then be retrieved through a get() method call.
 * 
 * example:
 * AddEntityWorker worker = new AddEntityWorker(entity);
 * registerWorker(worker);
 * Object value = worker.get();
 *
 * @see DataManagerWorker
 * @see KernelProxy
 *
 * @author Gordon Murphy, gpmurphy@indiana.edu
 * @author Bryan Dawson, bdawson@indiana.edu
 */
public abstract class InterfaceWorker implements DataManagerWorker {
    private Object value;  // see getValue(), setValue()

    /**
     * Start a thread that will call the <code>construct</code> method
     * and then exit.
     */

    public InterfaceWorker() {       
        }

    /** 
     * Get the value produced by the worker thread, or null if it 
     * hasn't been constructed yet.
     */
    protected synchronized Object getValue() { 
	return value; 
    }

    /** 
     * Set the value produced by worker thread 
     */
    private synchronized void setValue(Object x) {
	value = x;
	try {
		notifyAll();
		}
	catch (IllegalMonitorStateException e) {
		e.printStackTrace();
		}
    }

    /** 
     * Compute the value to be returned by the <code>get</code> method. 
     */
    protected abstract Object construct();

    /**
     * Return the value created by the <code>construct</code> method.  
     * Returns null if either the constructing thread or
     * the current thread was interrupted before a value was produced.
     * 
     * @return the value created by the <code>construct</code> method
     */
    public synchronized Object get() {
            try {
               if (getValue() == null) {
			wait();
			}
                }
            catch (InterruptedException e) {
                Thread.currentThread().interrupt(); // propagate
		    e.printStackTrace();
                return null;
                }
		return getValue();
    }
    public void process() {
	Object object = construct();
	if (object == null) {
		object = new Boolean(true);
		}
	setValue(object);
	}
}
