/*
class org.datamanager.userinterface.cerulean.model.kernel.KernelProxy
KnownSpace: Cerulean

Authors:
~~~~~~~~
Gordon Murphy, gpmurphy@indiana.edu
Bryan Dawson, bdawson@indiana.edu
*/

////////////////////////////////////////////////////////////////////////////////

package org.datamanager.userinterface.cerulean.model.kernel;
import java.io.Serializable;

import java.util.Vector;
import org.datamanager.constraint.AndConstraint;
import org.datamanager.constraint.EntityConstraint;
import org.datamanager.constraint.EventTypeIsConstraint;
import org.datamanager.constraint.InterfaceNameContainsConstraint;
import org.datamanager.constraint.NameEqualsConstraint;
import org.datamanager.constraint.NotConstraint;
import org.datamanager.event.AddEntityEvent;
import org.datamanager.event.DataManagerEvent;
import org.datamanager.exception.DataManagerException;
import org.datamanager.kernel.DataManagerWorker;
import org.datamanager.kernel.Entity;
import org.datamanager.kernel.EntityValue;
import org.datamanager.kernel.EventHandler;
import org.datamanager.constraint.EventConstraint;
import org.datamanager.kernel.WorkerRegistry;
import org.datamanager.kernel.Pool;
import org.datamanager.kernel.EventGenerator;
import org.datamanager.userinterface.cerulean.model.InterfaceClusterManager;
import org.datamanager.userinterface.cerulean.model.InterfaceEntity;
import org.datamanager.userinterface.cerulean.model.data.storage.InterfaceEntityList;
import org.datamanager.userinterface.cerulean.model.kernel.AttachEntityWorker;
import org.datamanager.userinterface.cerulean.model.kernel.CreateEntityWorker;
import org.datamanager.userinterface.cerulean.model.kernel.DetachEntityWorker;
import org.datamanager.userinterface.cerulean.model.kernel.GetAttributesWorker;
import org.datamanager.userinterface.cerulean.model.kernel.GetBasesWorker;
import org.datamanager.userinterface.cerulean.model.kernel.GetEntityValueWorker;
import org.datamanager.userinterface.cerulean.model.kernel.GetNameWorker;
import org.datamanager.userinterface.cerulean.model.kernel.SearchWorker;
import org.datamanager.userinterface.cerulean.model.kernel.SetEntityValueWorker;
import org.datamanager.userinterface.cerulean.model.kernel.SubscriptionWorker;

////////////////////////////////////////////////////////////////////////////////
/**
 * KernelProxy serves as a wrapper around the functionality of the kernel.
 *  
 * @see java.lang.Thread
 * @see org.datamanager.userinterface.cerulean.model.InterfaceEntity
 * @see org.datamanager.userinterface.cerulean.model.InterfaceCluster
 * @see org.datamanager.userinterface.cerulean.model.data.storage.InterfaceEntityList 
 * @see org.datamanager.kernel.Entity
 * @see org.datamanager.kernel.EventConstraint
 *
 * @author Gordon Murphy gpmurphy@indiana.edu
 * @author Bryan Dawson bdawson@indiana.edu
 * 
 */
////////////////////////////////////////////////////////////////////////////////
public class KernelProxy implements EventHandler, Serializable {
	public static final String IGNORED_ENTITY = "IgnoredByInterface";
	private static KernelProxy instance = new KernelProxy();

////////////////////////////////////////////////////////////////////////////////
	/**
	 * Private constructor.  Use KernelProxy.getInstance() to retrieve an
	 * instance.
	 */
	private KernelProxy() {
		EventTypeIsConstraint constraint = new EventTypeIsConstraint(AddEntityEvent.class);
		registerWorker(new SubscriptionWorker(constraint, this));
		}

////////////////////////////////////////////////////////////////////////////////
	public static KernelProxy getInstance() {
		return instance;
		}

////////////////////////////////////////////////////////////////////////////////
	/**
	 * Returns the results of performing a search for all entities whose name
	 * contains the specified string.
	 * @param name the name to search for
	 * @return a list InterfaceEntities whose names match
	 */
	public InterfaceEntityList searchByName(String name) {
		return search(new InterfaceNameContainsConstraint(name));
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Searches for all Entities that match the given constraint.
 * @param constraint the constraint that entities must match
 * @return the results of the search
 */
	public InterfaceEntityList search(EntityConstraint constraint) {
		EntityConstraint ignoredNamesConstraint = new NotConstraint(getIgnoredByInterfaceConstraint());
		EntityConstraint newConstraint = new AndConstraint(ignoredNamesConstraint, constraint);
		SearchWorker worker = new SearchWorker(newConstraint);
		registerWorker(worker);
		InterfaceEntityList returnValue =(InterfaceEntityList)(worker.get());
		return returnValue;
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Removes an entity from the pool.
 * @param entity the entity to remove.
 */	
	public void addEntity(Entity entity) {
		AddEntityWorker worker = new AddEntityWorker(entity);
		registerWorker(worker);
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Removes an entity from the pool.
 * @param entity the entity to remove.
 */	
	public void removeEntity(Entity entity) {
		RemoveEntityWorker worker = new RemoveEntityWorker(entity);
		registerWorker(worker);
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Searches for all Entities that match the given constraint.
 * @param constraint the constraint that entities must match
 * @return the results of the search
 */
	public String getEntityName(Entity entity) {
		GetNameWorker worker = new GetNameWorker(entity);
		registerWorker(worker);
		String name = (String)(worker.get());
		return name;
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Attaches an Entity as an attribute to another Entity.
 * @param base the base Entity to which the attribute will be attached 
 * @param attribute the attribute to attach
 */
	public void attachEntity(Entity base, Entity attribute) {
		AttachEntityWorker worker = new AttachEntityWorker(base, attribute);
		registerWorker(worker);
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Detaches an Entity as an attribute from another Entity.
 * @param base the base Entity to which the attribute will be detached 
 * @param attribute the attribute to detach
 */
	public void detachEntity(Entity base, Entity attribute) {
		DetachEntityWorker worker = new DetachEntityWorker(base, attribute);
		registerWorker(worker);
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Creates a new Entity with the specified name.
 * @param name the name of the new Entity
 * @return the new Entity
 */
	public Entity createEntity(String name) {
		CreateEntityWorker worker = new CreateEntityWorker(name);
		registerWorker(worker);
		Entity createdEntity = (Entity)(worker.get());
		return createdEntity;
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Creates a new Entity with an attached entity specifying that it is to be 
 * ignored by the interface.
 * @param name the name of the new Entity
 * @return the new Entity
 */
	public Entity createAttribute(String name) {
		CreateAttributeWorker worker = new CreateAttributeWorker(name);
		registerWorker(worker);
		Entity createdEntity = (Entity)(worker.get());
		return createdEntity;
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Sets the specified entity's value to the specified value.
 * @param entity the entity whose value will be changed
 * @param value the value to be added
 */
	public void setEntityValue(Entity entity, EntityValue value) {
		SetEntityValueWorker worker = new SetEntityValueWorker(entity, value);
		registerWorker(worker);
		}
	
////////////////////////////////////////////////////////////////////////////////
/**
 * Sets the specified entity's value to the specified value.
 * @param entity the entity whose value will be changed
 * @param value the value to be added
 */
	public EntityValue getEntityValue(Entity entity) {
		GetEntityValueWorker worker = new GetEntityValueWorker(entity);
		registerWorker(worker);
		EntityValue entityValue = (EntityValue)(worker.get());
		return entityValue;
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Retrieve all the bases of an entity that match the given constraint.
 * @param entity the entity to find the bases of
 * @param constraint the constraint that the bases much match
 * @return the bases that match the constraint
 */
	public Entity[] getBases(Entity entity, EntityConstraint constraint) {
		GetBasesWorker worker = new GetBasesWorker(entity, constraint);
		registerWorker(worker);
		Entity[] entities = (Entity[])(worker.get());
		return entities;
		}
	
////////////////////////////////////////////////////////////////////////////////
/**
 * Retrieve all the attributes of an entity that match the given constraint.
 * @param entity the entity to find the attributes of
 * @param constraint the constraint that the attributes much match
 * @return the attributes that match the constraint
 */
	public Entity[] getAttributes(Entity entity, EntityConstraint constraint) {
		GetAttributesWorker worker = new GetAttributesWorker(entity, constraint);
		registerWorker(worker);
		Entity[] entities = (Entity[])(worker.get());
		return entities;
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Add a DataManagerEvent to the pool.  
 * @param event the event to add to the pool
 */
	public void addEvent(DataManagerEvent event) {
		AddEventWorker worker = new AddEventWorker(event);
		registerWorker(worker);
		}
	
////////////////////////////////////////////////////////////////////////////////
/**
 * Subscribe handler to the event described by the constraint.
 * @param constraint the constraint describing the event
 * @param handler the EventHandler to be called when an event happens
 */
	public void subscribeToEvent(EventConstraint constraint, EventHandler handler) {
		registerWorker(new SubscriptionWorker(constraint, handler));
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Subscribe handler to the event whose Class is class.
 * @param event the Class class of the Event to be subscribe to.
 * @param handler the EventHandler to be called when an event happens
 */
	public void subscribeToEvent(Class event, EventHandler handler) {
		EventTypeIsConstraint constraint = new EventTypeIsConstraint(event);
		this.subscribeToEvent(event, handler);
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Register a DataManagerWorker thread.  The thread is implicitly started.
 * @param worker the thread to register
 */
	public static void registerWorker(DataManagerWorker worker) {
		WorkerRegistry.getInstance().register(worker);
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Register a DataManagerWorker thread.  The thread is implicitly started.
 * @param worker the thread to register
 * @param the priority of the thread
 * @see java.lang.Thread
 */
	public static void registerWorker(DataManagerWorker worker, int priority) {
		WorkerRegistry.getInstance().register(worker, priority);
		}

////////////////////////////////////////////////////////////////////////////////
/**
 * Register a DataManagerWorker thread.  The thread is implicitly started.
 * @param worker the thread to register
 * @param the priority of the thread
 * @see java.lang.Thread
 */
	public EntityConstraint getIgnoredByInterfaceConstraint() {
		EntityConstraint constraint = new NameEqualsConstraint(IGNORED_ENTITY);
		return constraint;
		}

	public EventGenerator getEventGenerator() {
		EventGenerator generator = null;
		try {
			generator = Pool.getDefaultPool();
			}
		catch (DataManagerException e) {
			e.printStackTrace();
			}
		return generator;
		}

////////////////////////////////////////////////////////////////////////////////
	public void handle(DataManagerEvent event) {
		if (event instanceof AddEntityEvent) {
			try {
				AddEntityEvent entityEvent = (AddEntityEvent)event;
				Entity addedEntity = entityEvent.getAddedEntity();
				Entity[] attributes = addedEntity.getAttributes(getIgnoredByInterfaceConstraint());
				boolean isIgnored = (attributes.length >= 1);
				//This is called from a Kernel Thread, so we are safe...
				String name = addedEntity.getName();
				if (!isIgnored) {
					InterfaceEntity interfaceEntity = new InterfaceEntity(addedEntity);
					InterfaceClusterManager.getInstance().getNewEntitiesCluster().addEntity(interfaceEntity);
					}
				}
			catch (DataManagerException e) {
				e.printStackTrace();
				}
			}
		}
	}