package org.datamanager.userinterface.cerulean.model.tree;

import org.datamanager.userinterface.cerulean.model.tree.InterfaceNode;
import org.datamanager.userinterface.cerulean.model.InterfaceEntity;
import org.datamanager.userinterface.cerulean.model.InterfaceEntityPeerTable;
import org.datamanager.userinterface.cerulean.model.InterfaceEntityManager;
import org.datamanager.userinterface.cerulean.model.menu.InterfaceEntityPopupMenu;
import javax.swing.JTree;

/**
 *This is the class that is going to be used to hold the Entity for display on
 *the EntityTree.  It will inherit most of its functionality from
 *DefaultMutableTreeNode. Three other methods come from InterfaceNode, to 
 *allow for user interaction with the tree. 
 *
 *@author Bryan Dawson (bdawson@indiana.edu)
 *@author Gordon Murphy (gpmurphy@indiana.edu)
 *
 *@see org.datamanager.userinterface.cerulean.model.tree.EntityTreeModel
 *@see org.datamanager.userinterface.cerulean.model.tree.InterfaceNode
 *@see org.datamanager.userinterface.cerulean.model.InterfaceEntity
 *@see javax.swing.tree.DefaultMutableTreeNode
 */

//  Methods:
//  ~~~~~~~~
//  public EntityNode(InterfaceEntity)
//          This is just the only constructor, does nothing but set the entity
//          field to the given InterfaceEntity.
//  public String toString()
//          This is method is overriding DefaultMutableTreeNode's toString
//          method.  It has been changed so that it returns the following
//          message: "<Entity's name>"
//  public void display()
//          Sets the InterfaceEntity to opened.
//  public void select()
//          Sets the InterfaceEntity to selected.
//  public void handlePopup(JTree,int,int)
//          Displays a InterfaceEntityPopupMenu corresponding to this objects 
//  	    InterfaceEntity.
//////////////////////////////////////////////////////////////////

public class EntityNode extends InterfaceNode
   {
   //to this nodes entity
   private InterfaceEntity entity;
   
   //the popup menu corresponding to this entity
   private InterfaceEntityPopupMenu menu;

   /////////////////////////////////////////////////////////////
   /**
    *The only constructor, currently only sets the entity field
    *@param entity Sets the entity field
    */
   public EntityNode(InterfaceEntity entity)
      {
      this.entity = entity;
      }

   /////////////////////////////////////////////////////////////
   /**
    *This method is used by the EntityTree to display the Entity's name
    *@return The message "<Entity's name>"
    */
   public String toString()
      {
      return new String(entity.getName());
      }

   /////////////////////////////////////////////////////////
   /**
    *Will display the contents of the Entity.
    */
   public void display()
      {
      //fire the proper events into the pool, also will handle
      //displaying of the EntityValue.  See InterfaceEntityManager
      this.select();
      entity.setOpened(this);
      }

   //////////////////////////////////////////////////////////////
   /**
    *Sets the InterfaceEntity corresponing to this node as the selected 
    *InterfaceEntity.
    */
   public void select()
      {
	InterfaceEntityManager.getInstance().setAsSelectedEntity(entity);
      }
  
   /////////////////////////////////////////////////////////////////
   /**
    * This displays the InterfaceEntityPopupMenu corresponding to this node's
    *InterfaceEntity.
    *@param tree The component that the popup menu is displayed over.
    *@param x The x-coordinate of the mouse click
    *@param y The y-coordinate of the mouse click
    */
   public void handlePopup(JTree tree, int x, int y)
     {
      if(menu == null)
         menu = new InterfaceEntityPopupMenu(InterfaceEntityPeerTable.getInstance().getPeer(entity));
     menu.show(tree,x,y);
     }      

   }//end of class






