package org.datamanager.userinterface.cerulean.model.tree;

import org.datamanager.userinterface.cerulean.controller.tree.ClusterSelectionListener;
import org.datamanager.userinterface.cerulean.model.tree.EntityNode;
import org.datamanager.userinterface.cerulean.model.tree.ClusterNode;
import org.datamanager.userinterface.cerulean.model.data.storage.InterfaceEntityList;
import org.datamanager.userinterface.cerulean.model.InterfaceCluster;
import org.datamanager.userinterface.cerulean.model.InterfaceClusterManager;
import org.datamanager.userinterface.cerulean.model.InterfaceEntityManager;
import org.datamanager.userinterface.cerulean.model.InterfaceEntity;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.MutableTreeNode;
import java.util.Hashtable;
import javax.swing.JTree;
import javax.swing.tree.TreePath;
/**
 *This class is the model used by the class EntityTree.  It takes care of
 *updating the tree when a user selects or unselects a cluster.  It uses the
 *class ClusterSectionListener to "listen" for these events to happen.  It is
 *pass a ClusterObject by the ClusterSectionListener, of which it makes(or
 *destroys) the proper nodes of the tree.  The cluster is the parent node with
 *the childern being the entities of that cluster.
 *
 *@see org.datamanager.userinterface.cerulean.model.event.ClusterEvent
 *@see org.datamanager.userinterface.cerulean.view.tree.EntityTree
 *@see org.datamanager.userinterface.cerulean.model.InterfaceCluster      
 *@see org.datamanager.userinterface.cerulean.controller.tree.ClusterSelectionListener
 *@see org.datamanager.userinterface.cerulean.controller.tree.ClickHandler 
 *@see org.datamanager.userinterface.cerulean.model.tree.EntityNode
 *@see org.datamanager.userinterface.cerulean.model.tree.ClusterNode
 *
 *@author Bryan Dawson (bdawson@indiana.edu)
 *@author Gordon Murphy (gpmurphy@indiana.edu)
 */

//  Methods:
//  ~~~~~~~~
//  public EntityTreeModel()
//          Sets up the model.  Sets the listener for ClusterEvents.
//  public void addCluster(InterfaceCluster)
//          The InterfaceCluster is sent from the listener.  This method first
//          adds the cluster in as a sub-root of the tree.  It then adds each
//          entity as a child of the cluster.  The clusterNode is also added
//          to a hashtable, with the InterfaceCluster as the key, for future 
//          reference.
//  public void removeCluster(InterfaceCluster)
//          The InterfaceCluster is again sent from the listener.  This method
//  	    looks up the clusterNode from the hashtable, and then removes the
//  	    children and itself from the tree.
//  public void addEntitiesToCluster(InterfaceCluster,InterfaceEntityList)
//          Add the given entites to the given clusetr
//////////////////////////////////////////////////////////////
public class EntityTreeModel extends DefaultTreeModel
   {
   private static RootNode root = new RootNode();
   private ClusterSelectionListener listener;
   private int clusterCounter;
   private Hashtable clusterNodeTable;
   private JTree tree;

   //////////////////////////////////////////////////
       /**
	*The only constructor.
	*@param tree The tree that this is a model for.
	*/
       public EntityTreeModel(JTree tree)
        {
        super(root);
       
        //Setup the ClusterListener, to know when the tree is to be changed
        listener = new ClusterSelectionListener(this);
        InterfaceClusterManager.getInstance().addClusterListener(listener);
	InterfaceEntityManager.getInstance().addEntityListener(listener);
        //this will hold the clusterNodes for easy access
        clusterNodeTable = new Hashtable();
        
        this.tree = tree;

        clusterCounter = 0;
        }

   //////////////////////////////////////////////////
   /**Called by the listener when a cluster is selected
    * This method first adds the cluster in as a sub-root of the tree.  
    * It then adds each entity  as a child of the cluster.  The clusterNode
    * is also added to a  hashtable, with the InterfaceCluster as the key,
    * for future reference.
    *@param cluster sent from the listener
    */
   public void addCluster(InterfaceCluster cluster)
        {
        //get the InterfaceEntityList of this cluster,which holds all the entities
        InterfaceEntityList entities = cluster.getInterfaceEntityList();
  
	//insert the cluster as sub-node of root
        ClusterNode clusterNode = new ClusterNode(cluster);
        insertNodeInto(clusterNode, root, clusterCounter);
		
        //get ready for the next cluster to be selected 
        clusterCounter++;

        //put the clusterNode in the Hashtable, with InterfaceCluster as key
        clusterNodeTable.put(cluster,clusterNode);
	addEntitiesToCluster(cluster,entities);
        TreePath currentPath = new TreePath(clusterNode.getPath());
        tree.expandRow(0);
        tree.expandPath(currentPath);
        }

   ///////////////////////////////////////////////////////////
       /**
	* To be called by the listener when a cluster is unselected.
	*This method looks up the clusterNode from the hashtable, and 
	*then removes the children and itself from the tree.
	*@param cluster sent by the listener
	*/
   public void removeCluster(InterfaceCluster cluster)
        {
        //get the cluster node
        ClusterNode clusterNode = (ClusterNode)clusterNodeTable.get(cluster);
        
        MutableTreeNode parent = (MutableTreeNode)(clusterNode.getParent());
            if (parent != null) 
                removeNodeFromParent(clusterNode);
        
	clusterNodeTable.remove(cluster);
        
        tree.treeDidChange();
        
        //get ready for the next cluster.
        clusterCounter--;
        }

    //////////////////////////////////////////////////////////
    /**
     * The add given InterfaceEntites to the given InterfaceCluster
     *@param cluster The cluster the entities are to be added to
     *@param entities The list of entities to be added
     */
    public void addEntitiesToCluster(InterfaceCluster cluster, 
				     InterfaceEntityList entities)
       {
	   ClusterNode clusterNode = (ClusterNode)clusterNodeTable.get(cluster);
	   EntityNode currentEntityNode;
	   int entityCounter = clusterNode.getChildCount();
	   for(int i = 0 ; i < entities.size() ; i++)
	       {
		   InterfaceEntity currentEntity = entities.elementAt(i);
		   currentEntityNode = new EntityNode(currentEntity);
		   
		   insertNodeInto(currentEntityNode, clusterNode, entityCounter);

		   entityCounter++;
	       }
	   //expand the node
	   TreePath currentPath = new TreePath(clusterNode.getPath());
	   tree.expandRow(0);
	   tree.expandPath(currentPath);
       }
   /////////////////////////////////////////////////////////////////////
       /**
	* Determines if the cluster in being displayed in the tree.
	*@param cluster The cluster in question
	*@return whether the cluster is in the tree
	*/
   public boolean contains(InterfaceCluster cluster)
      {
      return clusterNodeTable.containsKey(cluster);
      }

   }//end of class
        


           






        




