package org.datamanager.userinterface.cerulean.model.undo;

import org.datamanager.userinterface.cerulean.model.undo.InterfaceUndoEdit;
import org.datamanager.userinterface.cerulean.model.event.UndoDiedListener;
import org.datamanager.userinterface.cerulean.sounds.SoundRequestHandler;
import javax.swing.undo.UndoManager;
import javax.swing.event.UndoableEditListener;
import javax.swing.event.UndoableEditEvent;
import java.util.Vector;
import java.util.Enumeration;

/**
 *This class is the heart of the history section for the Cerulean interface.
 *It keeps track of the InterfaceUndoEdit's (Anything that can be undone in
 *Cerulean) and lets any listener know when an edit has been added to the
 *manager or when one dies.  The HistoryListModel class is set up as a
 *listener for this class.
 *
 *@author Bryan Dawson (bdawson@indiana.edu)
 *@author Gordon Murphy (gpmurphy@indiana.edu)
 *
 *@see org.datamanager.userinterface.cerulean.model.undo.InterfaceUndoEdit
 *@see UndoDiedListener
 *@see javax.swing.undo.UndoManager
 *@see javax.swing.event.UndoableEditListener
 */
//  Methods (also see swing's UndoManager class):
//  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//  private InterfaceUndoManager()
//  	private so that only will  exist
//  public InterfaceUndoManager getInstance()
//  	returns the only instance of this class	
//  public synchronized void addUndoableEditListener(UndoableEditListener)
//  	add a listener for UndoEdit's.
//  public synchronized void removeUndoableEditListener(UndonableEditListener)
//  	remove a listener for UndoEdit's 
//  public synchronized void addUndoDiedListener(UndoDiedListener)
//  	add a listener for UndoEdit's deaths.
//  public synchronized void removeUndoDiedListener(UndoDiedListener)
//  	remove a listener for UndoEdit's deaths
//  public  void fireUndoDied(InterfaceUndoEdit)
//  	fire an UndoDieEvent to listeners
//  public void fireUndoableEditHappened(InterfaceUndoEdit)
//  	fire an UndoableEditEvent to listeners
//  public synchronized boolean addEdit(InterfaceUndoEdit)
//  	add the edit and also fire an event
//  public synchronized void redoTo(InterfaceUndoEdit)
//  public synchronized void undoTo(interfaceUndoEdit)
//         These methods do the same as the superclasses, and also play 
//         the sound
////////////////////////////////////////////////////////////////////////////

public class InterfaceUndoManager extends UndoManager
   {
   //the one and only InterfaceUndoManager
   private static InterfaceUndoManager manager = new InterfaceUndoManager();
   
   //Holds the listeners for UndoableEditListener
   private Vector editListeners;
   
   //Holds the listeners for UndoDieListener
   private Vector dieListeners;
   
   //To play the sounds through
   private SoundRequestHandler sounds;
   //////////////////////////////////////////////////////////////////
   /**
    * To create the one and only instance of this class
    */
   private InterfaceUndoManager()
      {
      super();
      setLimit(100);
      editListeners = new Vector();
      dieListeners = new Vector();
      
      //load the sounds
      sounds = SoundRequestHandler.getInstance();
      sounds.load("redo.au");
      sounds.load("undo.au");
      }
      
   //////////////////////////////////////////////////////////////////
   /**
    *Get the only instance of this class.
    */
   public static InterfaceUndoManager getInstance()
      {
      //if(manager == null)
      //	 manager = new InterfaceUndoManager();
      return manager;
      }

   
   /////////////////////////////////////////////////////////////////
   /**
    * Add a listener for IntefaceUndoEdit's to the vector
    *@param listener The listener to be added 
    */
   public synchronized void addUndoableEditListener(UndoableEditListener listener)
      {
      editListeners.addElement(listener);
      }
   /////////////////////////////////////////////////////////////////
   /** 
    *Remove a listener for UndoEdit's from the vector
    *@param listener The listener to be removed
    */           
   public synchronized void removeUndoableEditListener(UndoableEditListener listener)
      {
      editListeners.removeElement(listener);
      }  
   
   /////////////////////////////////////////////////////////////////
   /**
    * Add a listener for UndoDied's to the vector
    *@param listener The listener to be added.
    */
   public synchronized void addUndoDiedListener(UndoDiedListener listener)
      {
      dieListeners.addElement(listener);
      }

   /////////////////////////////////////////////////////////////////
   /**
    *Remove a listener for UndoDied's from the vector
    *@param listener The listener to be removed.
    */           
   public synchronized void removeUndoDiedListener(UndoDiedListener listener)
      {
      dieListeners.removeElement(listener);
      } 
      
   /////////////////////////////////////////////////////////////////
   /**
    *Fire an UndoDieEvent to listeners.  Calls the undoEditDied method of
    *the registered listeners.
    *@param edit This is the InterfaceUndoEdit that UndoDiedEvent is created with.
    */
   public  void fireUndoDied(InterfaceUndoEdit edit)
      {
      Enumeration listeners = dieListeners.elements();
      while(listeners.hasMoreElements())
      	 {
	 UndoDiedListener currentListener = (UndoDiedListener)listeners.nextElement();
      	 currentListener.undoEditDied(new UndoableEditEvent(this,edit));
      	 } 
      } 
      
   /////////////////////////////////////////////////////////////////
   /**
    *Fire an UndoableEditEvent to listeners.  Calls the undoableEditHappened 
    *method of all registered listeners.
    *@param edit The InterfaceUndoEdit that the UndoableEditEvent is created with.
    */
   public  void fireUndoableEditHappened(InterfaceUndoEdit edit)
      {
      for (int i =0; i <= editListeners.size() - 1; i++ ) {
	 UndoableEditListener currentListener = (UndoableEditListener)editListeners.elementAt(i);
      	 currentListener.undoableEditHappened(new UndoableEditEvent(this, edit));
      	 } 
      } 
      
   /////////////////////////////////////////////////////////////////
   /**
    *  Add an InterfaceUndoEdit to the manager, and fire an event to listeners
    *@param edit The InterfaceUndoEdit to be added the manager.
    */    
   public synchronized boolean addEdit(InterfaceUndoEdit edit)
      {
      boolean didWork = super.addEdit(edit);
      if(didWork)
      	  fireUndoableEditHappened(edit);
      return didWork;
      } 
   /////////////////////////////////////////////////////////////////
   /**
    * Undoes all of the edits, until it reachs the given InterfaceUndoEdit.
    * And then plays the sound.  See Swing's UndoManager (ie the superclass)
    * for more information of the workings of this method. 
    *@param edit The InterfaceUndoEdit to be undone.
    */    
   public synchronized void undoTo(InterfaceUndoEdit edit)
      {
      super.undoTo(edit);
      
      //play the sound
      sounds.play("undo.au");
      }
   /////////////////////////////////////////////////////////////////
   /**
    * Redoes all of the edits, until it reachs the given InterfaceUndoEdit.
    * And then plays the sound.  See Swing's UndoManager (ie the superclass)
    * for more information of the workings of this method. 
    *@param edit The InterfaceUndoEdit to be redone.
    */ 
   public synchronized void redoTo(InterfaceUndoEdit edit)
      {
      super.redoTo(edit);

      //play the sound
      sounds.play("redo.au");
      }                          	     	         	             
      
   }//end of class
				
		
