package org.datamanager.userinterface.cerulean.model.undo;

import javax.swing.undo.CannotUndoException;
import javax.swing.undo.CannotRedoException;
import org.datamanager.userinterface.cerulean.model.undo.InterfaceUndoEdit;
import org.datamanager.userinterface.cerulean.model.undo.InterfaceUndoManager;
import org.datamanager.userinterface.cerulean.model.InterfaceEntity;
import org.datamanager.userinterface.cerulean.model.data.storage.InterfaceEntityList;
import org.datamanager.userinterface.cerulean.model.InterfaceCluster;

/**
 *This class holds the necessary information to undo/redo an removal of a
 *InterfaceEntity from a InterfaceCluster.  It will be created by the
 *InterfaceEntity and passed to the InterfaceUndoManager.  It uses the Swing 
 *undo facilities to keep track of most of the state.  It recieves a good bit
 *of its functionality from Swing's AbstractUndoEdit. 
 *
 *@author Bryan Dawson (bdawson@indiana.edu)
 *@author Gordon Murphy (gpmurphy@indiana.edu)
 *
 *@see org.datamanager.userinterface.cerulean.model.undo.InterfaceUndoManager
 *@see org.datamanager.userinterface.cerulean.model.undo.InterfaceUndoEdit
 *@see org.datamanager.userinterface.cerulean.model.data.storage.InterfaceEntityList
 *@see org.datamanager.userinterface.cerulean.model.InterfaceCluster
 */
//  Methods:
//  ~~~~~~~~
//  public AddUndoEdit(InterfaceEntityList, InterfaceCluster)
//          This creates a AddEntityUndoEdit for the corresponding
//  	    InterfaceEntityList.  The InterfaceCluster represents the
//  	    InterfaceCluster that the InterfaceEntity was added to.
//          This is important, because it allows us to remove the
//  	    InterfaceEntity back when we undo.
//  public void undo() throws CannotUndoException
//          Adds the InterfaceEntityList back to its cluster.
//  public void Redo() throws CannotRedoException
//          Removes the InterfaceEntityList from the Cluster.
//  public String toString()
//          returns the proper name from the history list to represent the edit
//          correctly.
//  public void die()
//  	IMPORTANT:calls super.die()  otherwise it will not be removed from the 
//  	list correctly. Also sets up things to be garbage collected.
/////////////////////////////////////////////////////////////////////

public class RemoveEntityUndoEdit extends InterfaceUndoEdit
   {
   //This is the InterfaceEntity that we want to undo/redo the addition of
   private InterfaceEntityList entities;

   //This is the InterfaceCluster that the InterfaceEntity belonged to
   private InterfaceCluster cluster;
   
   //this will be the source for the EntityClusteringEvent's 
   private InterfaceUndoManager manager;

   ////////////////////////////////////////////////////////
   /**
    * Creates the undo edit for the removal of the corresponding
    * InterfaceEntityList to the InterfaceCluster given.
    *@param entities The InterfaceEntities that were removed.
    *@param cluster The cluster from which they were removed from. 
    */
   public RemoveEntityUndoEdit(InterfaceEntityList entities, InterfaceCluster cluster)
      {
      this.entities = entities;
      this.cluster = cluster;
      manager = InterfaceUndoManager.getInstance();
      }

   ////////////////////////////////////////////////////////
   /**
    *  Redoes the removing of the cluster.  Overriden from the
    *  superclass.  To insure that you do not get an exception
    *  you should allows call the method canRedo() which returns
    *  a boolean saying if this edit can be redone at this time.
    */
   public void redo() throws CannotRedoException
      {
      //make sure all of the proper fields are set
      super.redo();
      //read the InterfaceEntity to the InterfaceCluster from which it came.
      cluster.removeEntities(manager, entities, true);
      }

   ////////////////////////////////////////////////////////
   /**
    * Undoes the addition of the entity.  Overriden from the
    *  superclass.  To insure that you do not get an exception
    *  you should allows call the method canUndo() which returns
    *  a boolean saying if this edit can be undone at this time.
    */
   public void undo() throws CannotUndoException
      {
      //make sure all of the proper fields are set
      super.undo();
      //remove the InterfaceEntity to the InterfaceCluster from which it came.
      cluster.addEntities(manager, entities, true);
      }

   ////////////////////////////////////////////////////////
   /** 
    *Set the references for garbage collection. IMPORTANT: This method calls 
    *super.die() to make sure that it is removed from the HistoryList.
    */
   public void die()
      {
      super.die();
      entities = null;
      cluster = null;
      }
   
   ///////////////////////////////////////////////////////////
   /**
    * Returns the proper string to be displayed in the HistoryList
    *@return The string to be displayed.
    */   
   public String toString()
      {
      if(this.canUndo())
      	 return new String("Undo Removal of Entities Containing: ");// +
//			   entities.elementAt(0).getName());
      else
      	 return new String("Redo Removal of Entities Containing: ");// +
//			   entities.elementAt(0).getName());
      }  

   }//end of class

