
package org.datamanager.userinterface.cerulean.sounds;

import org.datamanager.userinterface.cerulean.sounds.SoundRequestHandler;
import org.datamanager.userinterface.cerulean.model.event.EntityAndClusterAdapter;
import org.datamanager.userinterface.cerulean.model.event.ClusterContentEvent;
import org.datamanager.userinterface.cerulean.model.event.ClusterEvent;
import org.datamanager.userinterface.cerulean.model.event.EntityEvent;

/**
 *This class catches events intended for EntityListeners and ClusterListeners
 *and plays the corresponding sounds.  Not all events that occur have sounds 
 *corresponding to them, which is why this class subclasses an adapter class.
 * It has a predefined set of sounds that it play; which are loaded and
 * played through the SoundRequestHandler.  
 *
 *@author Bryan Dawson (bdawson@indiana.edu)
 *@author Gordon Murphy (gpmurphy@indiana.edu)
 *
 *@see SoundRequestHandler
 *@see EntityAndClusterAdapter
 */
// Methods:
// ~~~~~~~~
// public AutomaticSoundPlayer()
//      This is the only constructor.  It loads the sounds for future use.
// public void entitiesRemovedFromCluster(ClusterContentEvent)
//      This plays "entityremoved.au" on removal of entities.
// public void clusterSelected(ClusterEvent)
//      This plays "clusterengaged.au" when a user clicks on an unselcted  
//      cluster tab
// public void clusterUnselected(ClusterEvent)
//      This plays "clusterdisengaed.au" when a user clicks on a selected
//      cluster tab
// public void entitySelected(EntityEvent)
//      This plays "entityselected.au" when a user selects a entity.
/////////////////////////////////////////////////////////////


public class AutomaticSoundPlayer extends EntityAndClusterAdapter
   {
       //The class that the userinterface uses to play sounds
       private SoundRequestHandler sounds;

       ////////////////////////////////////////////////////////////
       /**
	* This constructor loads the sounds to be used by this class  
	*  for future playing of sounds.
	*/
       public AutomaticSoundPlayer()
          {
	      sounds = SoundRequestHandler.getInstance();
	      sounds.load("entityselected.au");
	      sounds.load("clusterengaged.au");
	      sounds.load("clusterdisengaged.au");
	      sounds.load("removeentity.au");
	  }

       ////////////////////////////////////////////////////////////
       /**
	* This method is called when a group of entities are removed
	*  from a cluster. It plays the sound "entityremoved.au"
	*/
       public void entitiesRemovedFromCluster(ClusterContentEvent event)
          {
	      //play the sound
	      sounds.play("removeentity.au");
	  }
       
       ////////////////////////////////////////////////////////////
       /* This method is called when an unselected cluster is selected
       /  by the user.  It play the sound "clusterengaged.au"
       */
       public void clusterSelected(ClusterEvent event)
          {
	      //play the sound
	      sounds.play("clusterengaged.au");
	  }
       
       ////////////////////////////////////////////////////////////
       /**
	* This method is called when the selected cluster is unselected
	*  by the user.  It plays the sound "clusterdisengaged.au"
	*/
       public void clusterUnselected(ClusterEvent event)
          {
	      //play the sound
	      sounds.play("clusterdisengaged.au");
	  }

       ////////////////////////////////////////////////////////////
       /**
	* This method is called when the user selects an entity.
	*  It plays the sound "entityselected.au"
	*/
       public void entitySelected(EntityEvent event)
          {
	      //play the sound
	      sounds.play("entityselected.au");
	  }
     
    }//end of class

