/*
class org.datamanager.userinterface.cerulean.model.InterfaceClusterPane
KnownSpace: Cerulean

Authors:
~~~~~~~~
Gordon Murphy, gpmurphy@indiana.edu
Bryan Dawson, bdawson@indiana.edu
*/

package org.datamanager.userinterface.cerulean.view.component;

import org.datamanager.userinterface.cerulean.view.*;
import org.datamanager.userinterface.cerulean.view.icon.*;
import org.datamanager.userinterface.cerulean.view.layout.*;
import org.datamanager.userinterface.cerulean.controller.cluster.*;
import org.datamanager.userinterface.cerulean.model.*;
import org.datamanager.userinterface.cerulean.model.data.transfer.*;
import org.datamanager.userinterface.cerulean.model.data.storage.*;
import org.datamanager.tools.Debug;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.awt.geom.*;

/**
 * InterfaceClusterPane is an extension of LayeredPane.  InterfaceClusterComponents can
 * be added to the pane by calling one of the addCluster() methods.  
 * Components added will appear in a circular layout by default.  If a 
 * specific degree on the circle is given, the component will be placed
 * there, otherwise the cluster will be added in the following way:
 * starting at the 90 degree mark (the top middle) it will add clusters.
 * The next cluster to be added will then be placed at the 270 degree mark
 * (the bottom middle).  Following clusters will then be added in the same
 * manner, except either to the left or right of the previous ones.
 *
 * @see InterfaceCluster
 * @see InterfaceClusterComponent
 * @see InterfaceClusterManager
 * @see InterfaceClusterPaneClusterListener
 * @see EntityList
 * @see InterfaceEntityList
 *
 * @version 1.0
 * @author Gordon Murphy, gpmurphy@indiana.edu
 * @author Bryan Dawson, bdawson@indiana.edu 
 */
public class InterfaceClusterPane extends CirclePane  {
	private int topDegree, bottomDegree;
	private boolean isClusterOnTop;
	private static InterfaceCluster searchCluster;
////////////////////////////////////////////////////////////////////////////////
/**
 * Creates a new InterfaceClusterPane to be added to a CRootLayout.
 */
	public InterfaceClusterPane() {
		super();
		setLayout(new CircleLayout2());
		setBorder(BorderFactory.createEmptyBorder(60, 60, 60, 60));
		InterfaceClusterPaneClusterListener clusterListener;
		clusterListener = new  InterfaceClusterPaneClusterListener(this);
		InterfaceClusterManager.getInstance().addClusterListener(clusterListener);
		topDegree = 90;
		bottomDegree = 270;
		isClusterOnTop = true;
		}
////////////////////////////////////////////////////////////////////////////////
/**
 * Adds a new InterfaceCluster to the cluster pane and creates a new InterfaceClusterComponent
 * for it.
 * @param cluster the cluster to add
 */
	public void addCluster(InterfaceCluster cluster) {
		InterfaceClusterComponent clusterComponent = new InterfaceClusterComponent(cluster);
		int degrees = getClusterPlacement(clusterComponent);
		add(clusterComponent, new Integer(degrees));
		revalidate();
		repaint();
		}
////////////////////////////////////////////////////////////////////////////////
/**
 * Adds a new InterfaceCluster to the cluster pane at the specified angle and creates a
 * new InterfaceClusterComponent for it.
 * @param cluster the cluster to add
 * @param degrees the angle at which to add the cluster
 */
	public void addCluster(InterfaceCluster cluster, int degrees) {
		InterfaceClusterComponent clusterComponent = new InterfaceClusterComponent(cluster);
		add(clusterComponent, new Integer(degrees));
		}
////////////////////////////////////////////////////////////////////////////////
/**
 * Adds a new InterfaceClusterComponent with the given name at the specified angle.
 * @param name the name of the cluster to add
 * @param degrees the angle at which to add the cluster
 */
	public void addCluster(String name, int degrees) {
		add(new InterfaceClusterComponent(name), new Integer(degrees));
		}
////////////////////////////////////////////////////////////////////////////////
/**
 * Adds a new InterfaceClusterComponent with the given name at the specified angle and
 * painted the specified Paint.
 * @param name the name of the cluster to add
 * @param degrees the angle at which to add the cluster
 * @param paint the paint type to  make the cluster
 */
	public void addCluster(String name, int degrees, Paint paint) {
		InterfaceClusterComponent c = new InterfaceClusterComponent(name, paint);
		add(c, new Integer(degrees));
		}
////////////////////////////////////////////////////////////////////////////////
/**
 * Adds a new InterfaceClusterComponent with the given name at the specified angle and 
 * using the specified InterfaceEntityList
 * @param name the name of the cluster to add
 * @param degrees the angle at which to add the cluster
 * @param list the EntityList to add to a cluster
 */
	public void addCluster(String name, int degrees, InterfaceEntityList list) {
		InterfaceCluster cluster = new InterfaceCluster(name, list);
		InterfaceClusterComponent c = new InterfaceClusterComponent(cluster);
		add(c, new Integer(degrees));
		}
////////////////////////////////////////////////////////////////////////////////
/**
 * Adds a new InterfaceClusterComponent specialized for the search function
 */
	public void addSearchCluster() {
		//InterfaceClusterManager is dependent upon the name "search" in order
		//to find the recommended cluster.  If you wish to change the name, then
		//you must change the name in InterfaceClusterManager.getSearchCluster()
		GradientPaint specialPaint = new GradientPaint(0f,0f,Color.gray.brighter().brighter(),
										50f,50f,Color.gray.darker().darker());
		searchCluster = new InterfaceCluster("search");
		searchCluster.setDissolvable(false);
		InterfaceClusterComponent c = new InterfaceClusterComponent(
									 searchCluster,
									 specialPaint);
		add(c, new Integer(0));
		}
////////////////////////////////////////////////////////////////////////////////
/**
 * Adds a new InterfaceClusterComponent specialized for the Recommended function
 */
	public void addRecommendedCluster() {
		GradientPaint specialPaint = new GradientPaint(0f,0f,Color.gray.brighter().brighter(),
										50f,50f,Color.gray.darker().darker());

		//InterfaceClusterManager is dependent upon the name "recommended" in order
		//to find the recommended cluster.  If you wish to change the name, then
		//you must change the name in InterfaceClusterManager.getRecommendedCluster()
		InterfaceCluster recommendedCluster = new InterfaceCluster("recommended");
		recommendedCluster.setDissolvable(false);
		InterfaceClusterComponent c = new InterfaceClusterComponent(recommendedCluster,
									 specialPaint);
		add(c, new Integer(180));
		}
////////////////////////////////////////////////////////////////////////////////
/**
 * Adds a new InterfaceClusterComponent specialized for the New Entities function
 */
	public void addNewEntitiesCluster() {
		GradientPaint specialPaint = new GradientPaint(0f,0f,Color.gray.brighter().brighter(),
										50f,50f,Color.gray.darker().darker());
		//InterfaceClusterManager is dependent upon the name "recommended" in order
		//to find the recommended cluster.  If you wish to change the name, then
		//you must change the name in InterfaceClusterManager.getRecommendedCluster()
		InterfaceCluster newCluster = new InterfaceCluster("new");
		newCluster.setDissolvable(false);
		InterfaceClusterComponent c = new InterfaceClusterComponent(newCluster,
									 specialPaint);
		add(c, new Integer(270));
		}
////////////////////////////////////////////////////////////////////////////////
/**
 * Removes a InterfaceCluster and its related peer from the clusterPane
 */
	public void removeCluster(InterfaceCluster cluster) {
		InterfaceClusterPeerTable peerTable = InterfaceClusterPeerTable.getInstance();
		InterfaceClusterComponent clusterComponent = peerTable.getPeer(cluster);
		remove(clusterComponent);
		repaint();
		}
////////////////////////////////////////////////////////////////////////////////
	public int getClusterPlacement(InterfaceClusterComponent clusterComponent) {
		int numComponents = getComponentCount()+1;
		int degree;
		if (isClusterOnTop) degree = topDegree;
		else degree = bottomDegree;
		int returnDegree = degree;
		double componentWidth = (double)(clusterComponent.getPreferredSize().width);
		double componentHeight = (getPreferredSize().width)/2;
		double angle = Math.atan(componentWidth/componentHeight);
		angle = Math.toDegrees(angle);
		int intAngle = (int)angle;
		degree = degree + intAngle;
		Debug.assert("angle"+angle);
		Debug.assert("degree"+degree);
		if (isClusterOnTop) 
			topDegree = degree;
		else bottomDegree = degree;
		isClusterOnTop = !isClusterOnTop;
		return returnDegree;
		}
	}