package org.datamanager.userinterface.cerulean.view.undo;

import javax.swing.JPanel;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.ImageIcon;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.BorderLayout;
import java.awt.Container;
import org.datamanager.userinterface.cerulean.view.undo.HistoryList;
import org.datamanager.userinterface.cerulean.sounds.SoundRequestHandler;

/**
 *This class will be used to hold the History list for the
 *Cerulean interface.  The interesting part of this class
 *is that it is collapsible.  When the user clicks on the button, this class 
 *resets the position of the divider in its parent JSplitPane (based on its 
 *current state, collapsed or expanded).  It is designed with idea that it 
 *will be the bottom component of a JSplitPane.  If this changes (either to 
 *the top component, or not using a JSplitPane at all) then this class will 
 *have to be changed. 
 *
 *@author Bryan Dawson (<a href=mailto:bdawson@indiana.edu>bdawson@indiana.edu</a>)
 *@author Gordon Murphy (<a href=mailto:gpmurphy@indiana.edu>gpmurphy@indiana.edu</a>)
 *
 *@see HistoryList
 *@see JPanel
 */
// methods:
// ~~~~~~~~
// public HistoryPane()
// 	creates a new history panel with the default border color
// public HistoryPane(Color)
// 	creates a new history panel with the specified border color	
// public void actionPerformed(ActionEvent event)
// 	handles the pushing of the button.
// private void collapse()
// 	shrinks the panel
// private void expand()
// 	pops up the panel
// public void setBorderColor(Color)
// public Color getBorderColor()
// 	set and get the border color			
/////////////////////////////////////////////////////////////////////////


public class HistoryPane extends JPanel implements ActionListener
{
   //this button controls the up and down movement of the panel
   private JButton downAndUp;
   
   //lets the world know that this is the history
   private JLabel historyLabel;
   
   //keeps the color of the border
   private Color borderColor;
   
   //the border
   private JPanel border;
   
   //keeps the current state
   private boolean isUp;
   
   //the images for the button
   private ImageIcon upArrow;
   private ImageIcon downArrow;
   
   //used in seting up the dimensions of the component
   private Container container;
   
   //The actual history list and the scroll pane to contain it
   private JScrollPane scroll;
   private HistoryList historyList;
   
  /**
   *RGB 135,135,167
   */
   public static final Color DEFAULT_BORDER_COLOR = new Color(135,135,167);
   
  /**
   *This class is designed to be put in a JSplitPane, it also assumes that it
   * is going to be in the bottom 1/3 of the split pane.
   */
   public static final double PERCENTAGE_OF_SPLITPANE = .67;
   private JSplitPane splitPane;
 
   ///////////////////////////////////////////////////////////////////
  /**
   * Calls the other constructor with the given JSplitPane and the 
   *  DEFAULT_BORDER_COLOR.
   */  
   public HistoryPane(JSplitPane splitPane)
   	{
	this(DEFAULT_BORDER_COLOR, splitPane);
	} 
 
   //////////////////////////////////////////////////////////////////
  /**
   * Setups everything in the HistoryPane, with the given border color
   *  and JSplitPane.
   */
   public HistoryPane(Color borderColor, JSplitPane splitPane)
   	{   	
   	//make the elements   	
	upArrow = new ImageIcon("org/datamanager/userinterface/cerulean/view/undo/images/uparrow_c.gif");
	downArrow = new ImageIcon("org/datamanager/userinterface/cerulean/view/undo/images/downarrow_c.gif"); 
   	downAndUp = new JButton(downArrow);
   	historyLabel = new JLabel("HISTORY");
   	border = new JPanel();
   	historyList = new HistoryList();
   	scroll = new JScrollPane(historyList);
   	this.borderColor = borderColor;
   	this.splitPane = splitPane;
   		   	
   	downAndUp.addActionListener(this);
   	
  	//Set the sizes
	this.setOpaque(false);
	this.setMinimumSize(new Dimension(200,35));
   	this.setMaximumSize(new Dimension(200, 200));
   	this.setPreferredSize(this.getMaximumSize());
	
	border.setPreferredSize(new Dimension(200,35));
	border.setBackground(borderColor);
	historyLabel.setForeground(new Color(224,224,244));
	
        //setup the button
   	downAndUp.setBorderPainted(false);
   	downAndUp.setContentAreaFilled(false);
   
   	//add the button and label to the border
	border.setLayout(new BorderLayout());
   	border.add(downAndUp, BorderLayout.WEST);
       	border.add(historyLabel, BorderLayout.CENTER);
       	
       	//add the border
       	this.setLayout(new BorderLayout());
       	this.add(border, BorderLayout.NORTH);
       	this.add(scroll, BorderLayout.CENTER);	
   	
   	//set so that it starts up
   	isUp = true;

	//add the sounds
	SoundRequestHandler sounds = SoundRequestHandler.getInstance();
	sounds.load("collapsehistory.au");
	sounds.load("expandhistory.au");
   	}
   	
   ////////////////////////////////////////////////////////////// 
   /**
    *Determine if you are going up or down
    */	
   public void actionPerformed(ActionEvent event)
   	{
   	 if(isUp)
   	   this.collapse();
   	 else
   	   this.expand();
   	   
   	}  
   //////////////////////////////////////////////////////////////// 
  /**
   * shrink the history panel, set the image of the button,play the sound
   * ,and set our position in the JSplitPane
   */
  private void collapse()
   	{
     	downAndUp.setIcon(upArrow);
   	isUp = !isUp;
   	this.setPreferredSize(this.getMinimumSize());
   	
   	//Assuming that the HistoryPane is the bottom component, if it is 
   	//instead the Top component then you to change Maximum to Minimum
   	splitPane.setDividerLocation(splitPane.getMaximumDividerLocation());
   	revalidate();

	//play the sound
	SoundRequestHandler.getInstance().play("collapsehistory.au");
   	}
   	
   //////////////////////////////////////////////////////////////// 
  /**
   *Expand the history panel, set the image of the button, play the sound
   *  , and 
   *  set our position in the JSplitPane
   */
   private void expand()
   	{
      	downAndUp.setIcon(downArrow);
   	isUp = !isUp;
   	this.setPreferredSize(this.getMaximumSize());
   	splitPane.setDividerLocation(HistoryPane.PERCENTAGE_OF_SPLITPANE);
   	revalidate();

	//play the sounds
	SoundRequestHandler.getInstance().play("expandhistory.au");
   	}
   	
   /////////////////////////////////////////////////////////////////
  /**
   * Set the border color property
   *@param borderColor Sets the border color to the given color, could be used in the furture for customizing the HistoryPane.
   */
   public void setBorderColor(Color borderColor)
   	{
   	this.borderColor = borderColor;
   	}
   /////////////////////////////////////////////////////////////////
  /**
   * get the border color property
   *@return The current color the border. By default RGB 135,135,167
   */
   public Color getBorderColor()
   	{
   	return borderColor;
   	}

  /////////////////////////////////////////////////////////////////   
  /**
   * The accessor for the isUp varible
   *@return Whether the HistoryPane is currently expaned(true) or collapsed(false)
   */
  public boolean isUp()
     {
     return isUp;
     }

    	
   }//end of class
   	   
   	
   	
   	
